@@ -1,4 +1,3 @@
-/* $Id$ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -21,6 +20,9 @@
 #include <pj/unicode.h>
 #include <pj/errno.h>
 #include <pj/assert.h>
+#include <pj/string.h>
+
+#if PJ_FILE_IO == PJ_FILE_IO_WIN32
 
 #include <windows.h>
 
@@ -28,6 +30,39 @@
 #   define INVALID_SET_FILE_POINTER     ((DWORD)-1)
 #endif
 
+static pj_status_t set_file_pointer(pj_oshandle_t fd,
+    pj_off_t offset,
+    pj_off_t* newPos,
+    DWORD dwMoveMethod)
+{
+#if defined(PJ_WIN32_WINPHONE8) && PJ_WIN32_WINPHONE8
+    LARGE_INTEGER liDistance, liNewPos;
+
+    liDistance.QuadPart = offset;
+    if (!SetFilePointerEx(fd, liDistance, &liNewPos, dwMoveMethod)) {
+        return PJ_RETURN_OS_ERROR(GetLastError());
+    }
+    *newPos = liNewPos.QuadPart;
+#else
+    DWORD dwNewPos;
+    LONG  hi32;
+
+    hi32 = (LONG)(offset >> 32);
+
+    dwNewPos = SetFilePointer(fd, (long)offset, &hi32, dwMoveMethod);
+    if (dwNewPos == (DWORD)INVALID_SET_FILE_POINTER) {
+        DWORD dwStatus = GetLastError();
+        if (dwStatus != 0)
+            return PJ_RETURN_OS_ERROR(dwStatus);
+        /* dwNewPos actually is not an error. */
+    }
+    *newPos = hi32;
+    *newPos = (*newPos << 32) + dwNewPos;
+#endif
+
+    return PJ_SUCCESS;
+}
+
 /**
  * Check for end-of-file condition on the specified descriptor.
  *
@@ -45,11 +80,11 @@ PJ_DEF(pj_status_t) pj_file_open( pj_pool_t *pool,
                                   unsigned flags,
                                   pj_oshandle_t *fd)
 {
-    PJ_DECL_UNICODE_TEMP_BUF(wpathname,256)
+    PJ_DECL_UNICODE_TEMP_BUF(wpathname, 256)
     HANDLE hFile;
     DWORD dwDesiredAccess = 0;
     DWORD dwShareMode = 0;
-    DWORD dwCreationDisposition = 0;
+    DWORD dwCreationDisposition = OPEN_EXISTING;
     DWORD dwFlagsAndAttributes = 0;
 
     PJ_UNUSED_ARG(pool);
@@ -60,24 +95,22 @@ PJ_DEF(pj_status_t) pj_file_open( pj_pool_t *pool,
         dwDesiredAccess |= GENERIC_WRITE;
         if ((flags & PJ_O_APPEND) == PJ_O_APPEND) {
 #if !defined(PJ_WIN32_WINCE) || !PJ_WIN32_WINCE
-	    /* FILE_APPEND_DATA is invalid on WM2003 and WM5, but it seems
-	     * to be working on WM6. All are tested on emulator though.
-	     * Removing this also seem to work (i.e. data is appended), so
-	     * I guess this flag is "optional".
-	     * See http://trac.pjsip.org/repos/ticket/825
-	     */
+            /* FILE_APPEND_DATA is invalid on WM2003 and WM5, but it seems
+             * to be working on WM6. All are tested on emulator though.
+             * Removing this also seem to work (i.e. data is appended), so
+             * I guess this flag is "optional".
+             * See https://github.com/pjsip/pjproject/issues/825
+             */
             dwDesiredAccess |= FILE_APPEND_DATA;
 #endif
-	    dwCreationDisposition |= OPEN_ALWAYS;
+            dwCreationDisposition = OPEN_ALWAYS;
         } else {
             dwDesiredAccess &= ~(FILE_APPEND_DATA);
-            dwCreationDisposition |= CREATE_ALWAYS;
+            dwCreationDisposition = CREATE_ALWAYS;
         }
     }
     if ((flags & PJ_O_RDONLY) == PJ_O_RDONLY) {
         dwDesiredAccess |= GENERIC_READ;
-        if (flags == PJ_O_RDONLY)
-            dwCreationDisposition |= OPEN_EXISTING;
     }
 
     if (dwDesiredAccess == 0) {
@@ -85,25 +118,45 @@ PJ_DEF(pj_status_t) pj_file_open( pj_pool_t *pool,
         return PJ_EINVAL;
     }
 
-    dwShareMode = FILE_SHARE_READ | FILE_SHARE_WRITE;
-    dwFlagsAndAttributes = FILE_ATTRIBUTE_NORMAL;
+    dwShareMode = FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE;
 
-    hFile = CreateFile(PJ_STRING_TO_NATIVE(pathname,wpathname,sizeof(wpathname)), 
-		       dwDesiredAccess, dwShareMode, NULL,
+    if ((flags & PJ_O_SEQUENTIAL) == PJ_O_SEQUENTIAL) {
+        dwFlagsAndAttributes |= FILE_FLAG_SEQUENTIAL_SCAN;
+    }
+    if ((flags & PJ_O_RANDOM) == PJ_O_RANDOM) {
+        dwFlagsAndAttributes |= FILE_FLAG_RANDOM_ACCESS;
+    }
+    if ((flags & PJ_O_ASYNC) == PJ_O_ASYNC) {
+        dwFlagsAndAttributes |= FILE_FLAG_OVERLAPPED;
+    }
+    if (!dwFlagsAndAttributes) dwFlagsAndAttributes = FILE_ATTRIBUTE_NORMAL;
+
+#if defined(PJ_WIN32_WINPHONE8) && PJ_WIN32_WINPHONE8  
+    hFile = CreateFile2(PJ_STRING_TO_NATIVE(pathname,
+                                            wpathname, sizeof(wpathname)),
+                        dwDesiredAccess, dwShareMode, dwCreationDisposition,
+                        NULL);
+#else
+    hFile = CreateFile(PJ_STRING_TO_NATIVE(pathname,
+                                           wpathname, sizeof(wpathname)),
+                       dwDesiredAccess, dwShareMode, NULL,
                        dwCreationDisposition, dwFlagsAndAttributes, NULL);
+#endif
+
     if (hFile == INVALID_HANDLE_VALUE) {
+        DWORD lastErr = GetLastError(); 
         *fd = 0;
-        return PJ_RETURN_OS_ERROR(GetLastError());
+        return lastErr == ERROR_FILE_NOT_FOUND ? PJ_ENOTFOUND : PJ_RETURN_OS_ERROR(lastErr);
     }
 
     if ((flags & PJ_O_APPEND) == PJ_O_APPEND) {
-	pj_status_t status;
+        pj_status_t status;
 
-	status = pj_file_setpos(hFile, 0, PJ_SEEK_END);
-	if (status != PJ_SUCCESS) {
-	    pj_file_close(hFile);
-	    return status;
-	}
+        status = pj_file_setpos(hFile, 0, PJ_SEEK_END);
+        if (status != PJ_SUCCESS) {
+            pj_file_close(hFile);
+            return status;
+        }
     }
 
     *fd = hFile;
@@ -180,8 +233,7 @@ PJ_DEF(pj_status_t) pj_file_setpos( pj_oshandle_t fd,
                                     enum pj_file_seek_type whence)
 {
     DWORD dwMoveMethod;
-    DWORD dwNewPos;
-    LONG  hi32;
+    pj_off_t newPos;
 
     if (whence == PJ_SEEK_SET)
         dwMoveMethod = FILE_BEGIN;
@@ -194,34 +246,13 @@ PJ_DEF(pj_status_t) pj_file_setpos( pj_oshandle_t fd,
         return PJ_EINVAL;
     }
 
-    hi32 = (LONG)(offset >> 32);
-    dwNewPos = SetFilePointer(fd, (long)offset, &hi32, dwMoveMethod);
-    if (dwNewPos == (DWORD)INVALID_SET_FILE_POINTER) {
-        DWORD dwStatus = GetLastError();
-        if (dwStatus != 0)
-            return PJ_RETURN_OS_ERROR(dwStatus);
-        /* dwNewPos actually is not an error. */
-    }
-
-    return PJ_SUCCESS;
+    return set_file_pointer(fd, offset, &newPos, dwMoveMethod);
 }
 
 PJ_DEF(pj_status_t) pj_file_getpos( pj_oshandle_t fd,
                                     pj_off_t *pos)
 {
-    LONG hi32 = 0;
-    DWORD lo32;
-
-    lo32 = SetFilePointer(fd, 0, &hi32, FILE_CURRENT);
-    if (lo32 == (DWORD)INVALID_SET_FILE_POINTER) {
-        DWORD dwStatus = GetLastError();
-        if (dwStatus != 0)
-            return PJ_RETURN_OS_ERROR(dwStatus);
-    }
-
-    *pos = hi32;
-    *pos = (*pos << 32) + lo32;
-    return PJ_SUCCESS;
+    return set_file_pointer(fd, 0, pos, FILE_CURRENT);
 }
 
 PJ_DEF(pj_status_t) pj_file_flush(pj_oshandle_t fd)
@@ -231,10 +262,11 @@ PJ_DEF(pj_status_t) pj_file_flush(pj_oshandle_t fd)
     rc = FlushFileBuffers(fd);
 
     if (!rc) {
-	DWORD dwStatus = GetLastError();
+        DWORD dwStatus = GetLastError();
         if (dwStatus != 0)
             return PJ_RETURN_OS_ERROR(dwStatus);
     }
 
     return PJ_SUCCESS;
 }
+#endif //PJ_FILE_IO == PJ_FILE_IO_WIN32
