@@ -1,4 +1,3 @@
-/* $Id$ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -41,6 +40,11 @@
 #include <pj/errno.h>
 #include <pj/rand.h>
 
+
+/* Only build when the backend is using select(). */
+#if PJ_IOQUEUE_IMP == PJ_IOQUEUE_IMP_SELECT
+
+
 /* Now that we have access to OS'es <sys/select>, lets check again that
  * PJ_IOQUEUE_MAX_HANDLES is not greater than FD_SETSIZE
  */
@@ -81,15 +85,15 @@
  */
 /*
 #if defined(PJ_DEBUG) && PJ_DEBUG != 0
-#  define VALIDATE_FD_SET		1
+#  define VALIDATE_FD_SET               1
 #else
-#  define VALIDATE_FD_SET		0
+#  define VALIDATE_FD_SET               0
 #endif
 */
 #define VALIDATE_FD_SET     0
 
 #if 0
-#  define TRACE__(args)	PJ_LOG(3,args)
+#  define TRACE__(args) PJ_LOG(3,args)
 #else
 #  define TRACE__(args)
 #endif
@@ -109,28 +113,34 @@ struct pj_ioqueue_t
 {
     DECLARE_COMMON_IOQUEUE
 
-    unsigned		max, count;	/* Max and current key count	    */
-    int			nfds;		/* The largest fd value (for select)*/
-    pj_ioqueue_key_t	active_list;	/* List of active keys.		    */
-    pj_fd_set_t		rfdset;
-    pj_fd_set_t		wfdset;
+    unsigned            max, count;     /* Max and current key count        */
+    int                 nfds;           /* The largest fd value (for select)*/
+    pj_ioqueue_key_t    active_list;    /* List of active keys.             */
+    pj_fd_set_t         rfdset;
+    pj_fd_set_t         wfdset;
 #if PJ_HAS_TCP
-    pj_fd_set_t		xfdset;
+    pj_fd_set_t         xfdset;
 #endif
 
 #if PJ_IOQUEUE_HAS_SAFE_UNREG
-    pj_mutex_t	       *ref_cnt_mutex;
-    pj_ioqueue_key_t	closing_list;
-    pj_ioqueue_key_t	free_list;
+    pj_mutex_t         *ref_cnt_mutex;
+    pj_ioqueue_key_t    closing_list;
+    pj_ioqueue_key_t    free_list;
 #endif
 };
 
 /* Proto */
 #if defined(PJ_IPHONE_OS_HAS_MULTITASKING_SUPPORT) && \
-	    PJ_IPHONE_OS_HAS_MULTITASKING_SUPPORT!=0
+            PJ_IPHONE_OS_HAS_MULTITASKING_SUPPORT!=0
 static pj_status_t replace_udp_sock(pj_ioqueue_key_t *h);
 #endif
 
+#if defined(PJ_HAS_SSL_SOCK) && PJ_HAS_SSL_SOCK != 0 && \
+    (PJ_SSL_SOCK_IMP == PJ_SSL_SOCK_IMP_APPLE)
+    /* Call SSL Network framework poll */
+pj_status_t ssl_network_event_poll();
+#endif
+
 /* Include implementation for common abstraction after we declare
  * pj_ioqueue_key_t and pj_ioqueue_t.
  */
@@ -160,9 +170,9 @@ static void rescan_fdset(pj_ioqueue_t *ioqueue)
     int max = 0;
 
     while (key != &ioqueue->active_list) {
-	if (key->fd > max)
-	    max = key->fd;
-	key = key->next;
+        if (key->fd > max)
+            max = key->fd;
+        key = key->next;
     }
 
     ioqueue->nfds = max;
@@ -180,13 +190,27 @@ static void rescan_fdset(pj_ioqueue_t *ioqueue)
  *
  * Create select ioqueue.
  */
-PJ_DEF(pj_status_t) pj_ioqueue_create( pj_pool_t *pool, 
+PJ_DEF(pj_status_t) pj_ioqueue_create( pj_pool_t *pool,
                                        pj_size_t max_fd,
                                        pj_ioqueue_t **p_ioqueue)
+{
+    return pj_ioqueue_create2(pool, max_fd, NULL, p_ioqueue);
+}
+
+
+/*
+ * pj_ioqueue_create2()
+ *
+ * Create select ioqueue.
+ */
+PJ_DEF(pj_status_t) pj_ioqueue_create2(pj_pool_t *pool,
+                                       pj_size_t max_fd,
+                                       const pj_ioqueue_cfg *cfg,
+                                       pj_ioqueue_t **p_ioqueue)
 {
     pj_ioqueue_t *ioqueue;
     pj_lock_t *lock;
-    unsigned i;
+    pj_size_t i;
     pj_status_t rc;
 
     /* Check that arguments are valid. */
@@ -202,6 +226,10 @@ PJ_DEF(pj_status_t) pj_ioqueue_create( pj_pool_t *pool,
     ioqueue = PJ_POOL_ALLOC_T(pool, pj_ioqueue_t);
     ioqueue_init(ioqueue);
 
+    if (cfg)
+        pj_memcpy(&ioqueue->cfg, cfg, sizeof(*cfg));
+    else
+        pj_ioqueue_cfg_default(&ioqueue->cfg);
     ioqueue->max = (unsigned)max_fd;
     ioqueue->count = 0;
     PJ_FD_ZERO(&ioqueue->rfdset);
@@ -224,7 +252,7 @@ PJ_DEF(pj_status_t) pj_ioqueue_create( pj_pool_t *pool,
      */
     rc = pj_mutex_create_simple(pool, NULL, &ioqueue->ref_cnt_mutex);
     if (rc != PJ_SUCCESS)
-	return rc;
+        return rc;
 
 
     /* Init key list */
@@ -234,29 +262,29 @@ PJ_DEF(pj_status_t) pj_ioqueue_create( pj_pool_t *pool,
 
     /* Pre-create all keys according to max_fd */
     for (i=0; i<max_fd; ++i) {
-	pj_ioqueue_key_t *key;
+        pj_ioqueue_key_t *key;
 
-	key = PJ_POOL_ALLOC_T(pool, pj_ioqueue_key_t);
-	key->ref_count = 0;
-	rc = pj_lock_create_recursive_mutex(pool, NULL, &key->lock);
-	if (rc != PJ_SUCCESS) {
-	    key = ioqueue->free_list.next;
-	    while (key != &ioqueue->free_list) {
-		pj_lock_destroy(key->lock);
-		key = key->next;
-	    }
-	    pj_mutex_destroy(ioqueue->ref_cnt_mutex);
-	    return rc;
-	}
+        key = PJ_POOL_ALLOC_T(pool, pj_ioqueue_key_t);
+        key->ref_count = 0;
+        rc = pj_lock_create_recursive_mutex(pool, NULL, &key->lock);
+        if (rc != PJ_SUCCESS) {
+            key = ioqueue->free_list.next;
+            while (key != &ioqueue->free_list) {
+                pj_lock_destroy(key->lock);
+                key = key->next;
+            }
+            pj_mutex_destroy(ioqueue->ref_cnt_mutex);
+            return rc;
+        }
 
-	pj_list_push_back(&ioqueue->free_list, key);
+        pj_list_push_back(&ioqueue->free_list, key);
     }
 #endif
 
     /* Create and init ioqueue mutex */
     rc = pj_lock_create_simple_mutex(pool, "ioq%p", &lock);
     if (rc != PJ_SUCCESS)
-	return rc;
+        return rc;
 
     rc = pj_ioqueue_set_lock(ioqueue, lock, PJ_TRUE);
     if (rc != PJ_SUCCESS)
@@ -285,20 +313,20 @@ PJ_DEF(pj_status_t) pj_ioqueue_destroy(pj_ioqueue_t *ioqueue)
     /* Destroy reference counters */
     key = ioqueue->active_list.next;
     while (key != &ioqueue->active_list) {
-	pj_lock_destroy(key->lock);
-	key = key->next;
+        pj_lock_destroy(key->lock);
+        key = key->next;
     }
 
     key = ioqueue->closing_list.next;
     while (key != &ioqueue->closing_list) {
-	pj_lock_destroy(key->lock);
-	key = key->next;
+        pj_lock_destroy(key->lock);
+        key = key->next;
     }
 
     key = ioqueue->free_list.next;
     while (key != &ioqueue->free_list) {
-	pj_lock_destroy(key->lock);
-	key = key->next;
+        pj_lock_destroy(key->lock);
+        key = key->next;
     }
 
     pj_mutex_destroy(ioqueue->ref_cnt_mutex);
@@ -314,11 +342,11 @@ PJ_DEF(pj_status_t) pj_ioqueue_destroy(pj_ioqueue_t *ioqueue)
  * Register socket handle to ioqueue.
  */
 PJ_DEF(pj_status_t) pj_ioqueue_register_sock2(pj_pool_t *pool,
-					      pj_ioqueue_t *ioqueue,
-					      pj_sock_t sock,
-					      pj_grp_lock_t *grp_lock,
-					      void *user_data,
-					      const pj_ioqueue_callback *cb,
+                                              pj_ioqueue_t *ioqueue,
+                                              pj_sock_t sock,
+                                              pj_grp_lock_t *grp_lock,
+                                              void *user_data,
+                                              const pj_ioqueue_callback *cb,
                                               pj_ioqueue_key_t **p_key)
 {
     pj_ioqueue_key_t *key = NULL;
@@ -334,11 +362,22 @@ PJ_DEF(pj_status_t) pj_ioqueue_register_sock2(pj_pool_t *pool,
     PJ_ASSERT_RETURN(pool && ioqueue && sock != PJ_INVALID_SOCKET &&
                      cb && p_key, PJ_EINVAL);
 
+    /* On platforms with fd_set containing fd bitmap such as *nix family,
+     * avoid potential memory corruption caused by select() when given
+     * an fd that is higher than FD_SETSIZE.
+     */
+    if (sizeof(fd_set) < FD_SETSIZE && sock >= FD_SETSIZE) {
+        PJ_LOG(4, ("pjlib", "Failed to register socket to ioqueue because "
+                            "socket fd is too big (fd=%ld/FD_SETSIZE=%d)",
+                            sock, FD_SETSIZE));
+        return PJ_ETOOBIG;
+    }
+
     pj_lock_acquire(ioqueue->lock);
 
     if (ioqueue->count >= ioqueue->max) {
         rc = PJ_ETOOMANY;
-	goto on_return;
+        goto on_return;
     }
 
     /* If safe unregistration (PJ_IOQUEUE_HAS_SAFE_UNREG) is used, get
@@ -351,8 +390,8 @@ PJ_DEF(pj_status_t) pj_ioqueue_register_sock2(pj_pool_t *pool,
 
     pj_assert(!pj_list_empty(&ioqueue->free_list));
     if (pj_list_empty(&ioqueue->free_list)) {
-	rc = PJ_ETOOMANY;
-	goto on_return;
+        rc = PJ_ETOOMANY;
+        goto on_return;
     }
 
     key = ioqueue->free_list.next;
@@ -363,8 +402,8 @@ PJ_DEF(pj_status_t) pj_ioqueue_register_sock2(pj_pool_t *pool,
 
     rc = ioqueue_init_key(pool, ioqueue, key, sock, grp_lock, user_data, cb);
     if (rc != PJ_SUCCESS) {
-	key = NULL;
-	goto on_return;
+        key = NULL;
+        goto on_return;
     }
 
     /* Set socket to nonblocking. */
@@ -377,7 +416,7 @@ PJ_DEF(pj_status_t) pj_ioqueue_register_sock2(pj_pool_t *pool,
     if (ioctl(sock, FIONBIO, &value)) {
 #endif
         rc = pj_get_netos_error();
-	goto on_return;
+        goto on_return;
     }
 
 
@@ -391,8 +430,8 @@ PJ_DEF(pj_status_t) pj_ioqueue_register_sock2(pj_pool_t *pool,
 on_return:
     /* On error, socket may be left in non-blocking mode. */
     if (rc != PJ_SUCCESS) {
-	if (key && key->grp_lock)
-	    pj_grp_lock_dec_ref_dbg(key->grp_lock, "ioqueue", 0);
+        if (key && key->grp_lock)
+            pj_grp_lock_dec_ref_dbg(key->grp_lock, "ioqueue", 0);
     }
     *p_key = key;
     pj_lock_release(ioqueue->lock);
@@ -401,11 +440,11 @@ on_return:
 }
 
 PJ_DEF(pj_status_t) pj_ioqueue_register_sock( pj_pool_t *pool,
-					      pj_ioqueue_t *ioqueue,
-					      pj_sock_t sock,
-					      void *user_data,
-					      const pj_ioqueue_callback *cb,
-					      pj_ioqueue_key_t **p_key)
+                                              pj_ioqueue_t *ioqueue,
+                                              pj_sock_t sock,
+                                              void *user_data,
+                                              const pj_ioqueue_callback *cb,
+                                              pj_ioqueue_key_t **p_key)
 {
     return pj_ioqueue_register_sock2(pool, ioqueue, sock, NULL, user_data,
                                      cb, p_key);
@@ -432,15 +471,15 @@ static void decrement_counter(pj_ioqueue_key_t *key)
     --key->ref_count;
     if (key->ref_count == 0) {
 
-	pj_assert(key->closing == 1);
-	pj_gettickcount(&key->free_time);
-	key->free_time.msec += PJ_IOQUEUE_KEY_FREE_DELAY;
-	pj_time_val_normalize(&key->free_time);
+        pj_assert(key->closing == 1);
+        pj_gettickcount(&key->free_time);
+        key->free_time.msec += PJ_IOQUEUE_KEY_FREE_DELAY;
+        pj_time_val_normalize(&key->free_time);
 
-	pj_list_erase(key);
-	pj_list_push_back(&key->ioqueue->closing_list, key);
-	/* Rescan fdset to get max descriptor */
-	rescan_fdset(key->ioqueue);
+        pj_list_erase(key);
+        pj_list_push_back(&key->ioqueue->closing_list, key);
+        /* Rescan fdset to get max descriptor */
+        rescan_fdset(key->ioqueue);
     }
     pj_mutex_unlock(key->ioqueue->ref_cnt_mutex);
     pj_lock_release(key->ioqueue->lock);
@@ -467,23 +506,42 @@ PJ_DEF(pj_status_t) pj_ioqueue_unregister( pj_ioqueue_key_t *key)
      */
     pj_ioqueue_lock_key(key);
 
+    /* Best effort to avoid double key-unregistration */
+    if (IS_CLOSING(key)) {
+        pj_ioqueue_unlock_key(key);
+        return PJ_SUCCESS;
+    }
+
     /* Also lock ioqueue */
     pj_lock_acquire(ioqueue->lock);
 
-    pj_assert(ioqueue->count > 0);
-    --ioqueue->count;
+    /* Avoid "negative" ioqueue count */
+    if (ioqueue->count > 0) {
+        --ioqueue->count;
+    } else {
+        /* If this happens, very likely there is double unregistration
+         * of a key.
+         */
+        pj_assert(!"Bad ioqueue count in key unregistration!");
+        PJ_LOG(1,(THIS_FILE, "Bad ioqueue count in key unregistration!"));
+    }
+
 #if !PJ_IOQUEUE_HAS_SAFE_UNREG
     /* Ticket #520, key will be erased more than once */
     pj_list_erase(key);
 #endif
-    PJ_FD_CLR(key->fd, &ioqueue->rfdset);
-    PJ_FD_CLR(key->fd, &ioqueue->wfdset);
+
+    /* Remove socket from sets and close socket. */
+    if (key->fd != PJ_INVALID_SOCKET) {
+        PJ_FD_CLR(key->fd, &ioqueue->rfdset);
+        PJ_FD_CLR(key->fd, &ioqueue->wfdset);
 #if PJ_HAS_TCP
-    PJ_FD_CLR(key->fd, &ioqueue->xfdset);
+        PJ_FD_CLR(key->fd, &ioqueue->xfdset);
 #endif
 
-    /* Close socket. */
-    pj_sock_close(key->fd);
+        pj_sock_close(key->fd);
+        key->fd = PJ_INVALID_SOCKET;
+    }
 
     /* Clear callback */
     key->cb.on_accept_complete = NULL;
@@ -496,43 +554,23 @@ PJ_DEF(pj_status_t) pj_ioqueue_unregister( pj_ioqueue_key_t *key)
      */
     pj_lock_release(ioqueue->lock);
 
-#if PJ_IOQUEUE_HAS_SAFE_UNREG
     /* Mark key is closing. */
     key->closing = 1;
 
+    pj_ioqueue_unlock_key(key);
+
+#if PJ_IOQUEUE_HAS_SAFE_UNREG
     /* Decrement counter. */
     decrement_counter(key);
+#else
+    /* Destroy the key lock */
+    pj_lock_destroy(key->lock);
+#endif
 
     /* Done. */
     if (key->grp_lock) {
-	/* just dec_ref and unlock. we will set grp_lock to NULL
-	 * elsewhere */
-	pj_grp_lock_t *grp_lock = key->grp_lock;
-	// Don't set grp_lock to NULL otherwise the other thread
-	// will crash. Just leave it as dangling pointer, but this
-	// should be safe
-	//key->grp_lock = NULL;
-	pj_grp_lock_dec_ref_dbg(grp_lock, "ioqueue", 0);
-	pj_grp_lock_release(grp_lock);
-    } else {
-	pj_ioqueue_unlock_key(key);
+        pj_grp_lock_dec_ref_dbg(key->grp_lock, "ioqueue", 0);
     }
-#else
-    if (key->grp_lock) {
-	/* set grp_lock to NULL and unlock */
-	pj_grp_lock_t *grp_lock = key->grp_lock;
-	// Don't set grp_lock to NULL otherwise the other thread
-	// will crash. Just leave it as dangling pointer, but this
-	// should be safe
-	//key->grp_lock = NULL;
-	pj_grp_lock_dec_ref_dbg(grp_lock, "ioqueue", 0);
-	pj_grp_lock_release(grp_lock);
-    } else {
-	pj_ioqueue_unlock_key(key);
-    }
-
-    pj_lock_destroy(key->lock);
-#endif
 
     return PJ_SUCCESS;
 }
@@ -543,9 +581,9 @@ PJ_DEF(pj_status_t) pj_ioqueue_unregister( pj_ioqueue_key_t *key)
  */
 #if VALIDATE_FD_SET
 static void validate_sets(const pj_ioqueue_t *ioqueue,
-			  const pj_fd_set_t *rfdset,
-			  const pj_fd_set_t *wfdset,
-			  const pj_fd_set_t *xfdset)
+                          const pj_fd_set_t *rfdset,
+                          const pj_fd_set_t *wfdset,
+                          const pj_fd_set_t *xfdset)
 {
     pj_ioqueue_key_t *key;
 
@@ -559,42 +597,42 @@ static void validate_sets(const pj_ioqueue_t *ioqueue,
 
     key = ioqueue->active_list.next;
     while (key != &ioqueue->active_list) {
-	if (!pj_list_empty(&key->read_list)
+        if (!pj_list_empty(&key->read_list)
 #if defined(PJ_HAS_TCP) && PJ_HAS_TCP != 0
-	    || !pj_list_empty(&key->accept_list)
+            || !pj_list_empty(&key->accept_list)
 #endif
-	    ) 
-	{
-	    pj_assert(PJ_FD_ISSET(key->fd, rfdset));
-	} 
-	else {
-	    pj_assert(PJ_FD_ISSET(key->fd, rfdset) == 0);
-	}
-	if (!pj_list_empty(&key->write_list)
+            ) 
+        {
+            pj_assert(PJ_FD_ISSET(key->fd, rfdset));
+        } 
+        else {
+            pj_assert(PJ_FD_ISSET(key->fd, rfdset) == 0);
+        }
+        if (!pj_list_empty(&key->write_list)
 #if defined(PJ_HAS_TCP) && PJ_HAS_TCP != 0
-	    || key->connecting
+            || key->connecting
 #endif
-	   )
-	{
-	    pj_assert(PJ_FD_ISSET(key->fd, wfdset));
-	}
-	else {
-	    pj_assert(PJ_FD_ISSET(key->fd, wfdset) == 0);
-	}
+           )
+        {
+            pj_assert(PJ_FD_ISSET(key->fd, wfdset));
+        }
+        else {
+            pj_assert(PJ_FD_ISSET(key->fd, wfdset) == 0);
+        }
 #if defined(PJ_HAS_TCP) && PJ_HAS_TCP != 0
-	if (key->connecting)
-	{
-	    pj_assert(PJ_FD_ISSET(key->fd, xfdset));
-	}
-	else {
-	    pj_assert(PJ_FD_ISSET(key->fd, xfdset) == 0);
-	}
+        if (key->connecting)
+        {
+            pj_assert(PJ_FD_ISSET(key->fd, xfdset));
+        }
+        else {
+            pj_assert(PJ_FD_ISSET(key->fd, xfdset) == 0);
+        }
 #endif /* PJ_HAS_TCP */
 
-	key = key->next;
+        key = key->next;
     }
 }
-#endif	/* VALIDATE_FD_SET */
+#endif  /* VALIDATE_FD_SET */
 
 
 /* ioqueue_remove_from_set()
@@ -603,21 +641,26 @@ static void validate_sets(const pj_ioqueue_t *ioqueue,
  * set for the specified event.
  */
 static void ioqueue_remove_from_set( pj_ioqueue_t *ioqueue,
+                                     pj_ioqueue_key_t *key,
+                                     enum ioqueue_event_type event_type )
+{
+    ioqueue_remove_from_set2(ioqueue, key, event_type);
+}
+
+static void ioqueue_remove_from_set2(pj_ioqueue_t *ioqueue,
                                      pj_ioqueue_key_t *key, 
-                                     enum ioqueue_event_type event_type)
+                                     unsigned event_types)
 {
     pj_lock_acquire(ioqueue->lock);
 
-    if (event_type == READABLE_EVENT)
+    if (event_types & READABLE_EVENT)
         PJ_FD_CLR((pj_sock_t)key->fd, &ioqueue->rfdset);
-    else if (event_type == WRITEABLE_EVENT)
+    if (event_types & WRITEABLE_EVENT)
         PJ_FD_CLR((pj_sock_t)key->fd, &ioqueue->wfdset);
 #if defined(PJ_HAS_TCP) && PJ_HAS_TCP!=0
-    else if (event_type == EXCEPTION_EVENT)
+    if (event_types & EXCEPTION_EVENT)
         PJ_FD_CLR((pj_sock_t)key->fd, &ioqueue->xfdset);
 #endif
-    else
-        pj_assert(0);
 
     pj_lock_release(ioqueue->lock);
 }
@@ -631,19 +674,24 @@ static void ioqueue_remove_from_set( pj_ioqueue_t *ioqueue,
 static void ioqueue_add_to_set( pj_ioqueue_t *ioqueue,
                                 pj_ioqueue_key_t *key,
                                 enum ioqueue_event_type event_type )
+{
+    ioqueue_add_to_set2(ioqueue, key, event_type);
+}
+
+static void ioqueue_add_to_set2(pj_ioqueue_t *ioqueue,
+                                pj_ioqueue_key_t *key,
+                                unsigned event_types )
 {
     pj_lock_acquire(ioqueue->lock);
 
-    if (event_type == READABLE_EVENT)
+    if (event_types & READABLE_EVENT)
         PJ_FD_SET((pj_sock_t)key->fd, &ioqueue->rfdset);
-    else if (event_type == WRITEABLE_EVENT)
+    if (event_types & WRITEABLE_EVENT)
         PJ_FD_SET((pj_sock_t)key->fd, &ioqueue->wfdset);
 #if defined(PJ_HAS_TCP) && PJ_HAS_TCP!=0
-    else if (event_type == EXCEPTION_EVENT)
+    if (event_types & EXCEPTION_EVENT)
         PJ_FD_SET((pj_sock_t)key->fd, &ioqueue->xfdset);
 #endif
-    else
-        pj_assert(0);
 
     pj_lock_release(ioqueue->lock);
 }
@@ -658,19 +706,19 @@ static void scan_closing_keys(pj_ioqueue_t *ioqueue)
     pj_gettickcount(&now);
     h = ioqueue->closing_list.next;
     while (h != &ioqueue->closing_list) {
-	pj_ioqueue_key_t *next = h->next;
+        pj_ioqueue_key_t *next = h->next;
 
-	pj_assert(h->closing != 0);
+        pj_assert(h->closing != 0);
 
-	if (PJ_TIME_VAL_GTE(now, h->free_time)) {
-	    pj_list_erase(h);
-	    // Don't set grp_lock to NULL otherwise the other thread
-	    // will crash. Just leave it as dangling pointer, but this
-	    // should be safe
-	    //h->grp_lock = NULL;
-	    pj_list_push_back(&ioqueue->free_list, h);
-	}
-	h = next;
+        if (PJ_TIME_VAL_GTE(now, h->free_time)) {
+            pj_list_erase(h);
+            // Don't set grp_lock to NULL otherwise the other thread
+            // will crash. Just leave it as dangling pointer, but this
+            // should be safe
+            //h->grp_lock = NULL;
+            pj_list_push_back(&ioqueue->free_list, h);
+        }
+        h = next;
     }
 }
 #endif
@@ -680,8 +728,8 @@ static void scan_closing_keys(pj_ioqueue_t *ioqueue)
 static pj_status_t replace_udp_sock(pj_ioqueue_key_t *h)
 {
     enum flags {
-	HAS_PEER_ADDR = 1,
-	HAS_QOS = 2
+        HAS_PEER_ADDR = 1,
+        HAS_QOS = 2
     };
     pj_sock_t old_sock, new_sock = PJ_INVALID_SOCKET;
     pj_sockaddr local_addr, rem_addr;
@@ -689,78 +737,128 @@ static pj_status_t replace_udp_sock(pj_ioqueue_key_t *h)
     pj_fd_set_t *fds[3];
     unsigned i, fds_cnt, flags=0;
     pj_qos_params qos_params;
-    unsigned msec;
-    pj_status_t status;
+    unsigned msec, msec2;
+    pj_status_t status = PJ_EUNKNOWN;
 
     pj_lock_acquire(h->ioqueue->lock);
 
     old_sock = h->fd;
 
+    fds_cnt = 0;
+    fds[fds_cnt++] = &h->ioqueue->rfdset;
+    fds[fds_cnt++] = &h->ioqueue->wfdset;
+#if PJ_HAS_TCP
+    fds[fds_cnt++] = &h->ioqueue->xfdset;
+#endif
+
     /* Can only replace UDP socket */
     pj_assert(h->fd_type == pj_SOCK_DGRAM());
 
-    PJ_LOG(4,(THIS_FILE, "Attempting to replace UDP socket %d", old_sock));
+    PJ_LOG(4,(THIS_FILE, "Attempting to replace UDP socket %ld", old_sock));
 
-    /* Investigate the old socket */
-    addr_len = sizeof(local_addr);
-    status = pj_sock_getsockname(old_sock, &local_addr, &addr_len);
-    if (status != PJ_SUCCESS)
-	goto on_error;
+    for (msec=20; (msec<1000 && status != PJ_SUCCESS); msec=msec*2)
+    {
+        if (msec > 20) {
+            PJ_LOG(4,(THIS_FILE, "Retry to replace UDP socket %ld", h->fd));
+            pj_thread_sleep(msec);
+        }
+        
+        if (old_sock != PJ_INVALID_SOCKET) {
+            /* Investigate the old socket */
+            addr_len = sizeof(local_addr);
+            status = pj_sock_getsockname(old_sock, &local_addr, &addr_len);
+            if (status != PJ_SUCCESS) {
+                PJ_PERROR(5,(THIS_FILE, status, "Error get socket name"));
+                continue;
+            }
+        
+            addr_len = sizeof(rem_addr);
+            status = pj_sock_getpeername(old_sock, &rem_addr, &addr_len);
+            if (status != PJ_SUCCESS) {
+                PJ_PERROR(5,(THIS_FILE, status, "Error get peer name"));
+            } else {
+                flags |= HAS_PEER_ADDR;
+            }
+
+            status = pj_sock_get_qos_params(old_sock, &qos_params);
+            if (status == PJ_STATUS_FROM_OS(EBADF) ||
+                status == PJ_STATUS_FROM_OS(EINVAL))
+            {
+                PJ_PERROR(5,(THIS_FILE, status, "Error get qos param"));
+                continue;
+            }
+        
+            if (status != PJ_SUCCESS) {
+                PJ_PERROR(5,(THIS_FILE, status, "Error get qos param"));
+            } else {
+                flags |= HAS_QOS;
+            }
+
+            /* We're done with the old socket, close it otherwise we'll get
+             * error in bind()
+             */
+            status = pj_sock_close(old_sock);
+            if (status != PJ_SUCCESS) {
+                PJ_PERROR(5,(THIS_FILE, status, "Error closing socket"));
+            }
+            
+            old_sock = PJ_INVALID_SOCKET;
+        }
+
+        /* Prepare the new socket */
+        status = pj_sock_socket(local_addr.addr.sa_family, PJ_SOCK_DGRAM, 0,
+                                &new_sock);
+        if (status != PJ_SUCCESS) {
+            PJ_PERROR(5,(THIS_FILE, status, "Error create socket"));
+            continue;
+        }
+
+        /* Even after the socket is closed, we'll still get "Address in use"
+         * errors, so force it with SO_REUSEADDR
+         */
+        val = 1;
+        status = pj_sock_setsockopt(new_sock, SOL_SOCKET, SO_REUSEADDR,
+                                    &val, sizeof(val));
+        if (status == PJ_STATUS_FROM_OS(EBADF) ||
+            status == PJ_STATUS_FROM_OS(EINVAL))
+        {
+            PJ_PERROR(5,(THIS_FILE, status, "Error set socket option"));
+            continue;
+        }
+
+        /* The loop is silly, but what else can we do? */
+        addr_len = pj_sockaddr_get_len(&local_addr);
+        for (msec2=20; msec2<1000 ; msec2=msec2*2) {
+            status = pj_sock_bind(new_sock, &local_addr, addr_len);
+            if (status != PJ_STATUS_FROM_OS(EADDRINUSE))
+                break;
+            PJ_LOG(4,(THIS_FILE, "Address is still in use, retrying.."));
+            pj_thread_sleep(msec2);
+        }
+
+        if (status != PJ_SUCCESS)
+            continue;
+
+        if (flags & HAS_QOS) {
+            status = pj_sock_set_qos_params(new_sock, &qos_params);
+            if (status == PJ_STATUS_FROM_OS(EINVAL)) {
+                PJ_PERROR(5,(THIS_FILE, status, "Error set qos param"));
+                continue;
+            }
+        }
+
+        if (flags & HAS_PEER_ADDR) {
+            status = pj_sock_connect(new_sock, &rem_addr, addr_len);
+            if (status != PJ_SUCCESS) {
+                PJ_PERROR(5,(THIS_FILE, status, "Error connect socket"));
+                continue;
+            }
+        }
+    }
     
-    addr_len = sizeof(rem_addr);
-    status = pj_sock_getpeername(old_sock, &rem_addr, &addr_len);
-    if (status == PJ_SUCCESS)
-	flags |= HAS_PEER_ADDR;
-
-    status = pj_sock_get_qos_params(old_sock, &qos_params);
-    if (status == PJ_SUCCESS)
-	flags |= HAS_QOS;
-
-    /* We're done with the old socket, close it otherwise we'll get
-     * error in bind()
-     */
-    pj_sock_close(old_sock);
-
-    /* Prepare the new socket */
-    status = pj_sock_socket(local_addr.addr.sa_family, PJ_SOCK_DGRAM, 0,
-			    &new_sock);
     if (status != PJ_SUCCESS)
-	goto on_error;
-
-    /* Even after the socket is closed, we'll still get "Address in use"
-     * errors, so force it with SO_REUSEADDR
-     */
-    val = 1;
-    status = pj_sock_setsockopt(new_sock, SOL_SOCKET, SO_REUSEADDR,
-				&val, sizeof(val));
-    if (status != PJ_SUCCESS)
-	goto on_error;
-
-    /* The loop is silly, but what else can we do? */
-    addr_len = pj_sockaddr_get_len(&local_addr);
-    for (msec=20; ; msec<1000? msec=msec*2 : 1000) {
-	status = pj_sock_bind(new_sock, &local_addr, addr_len);
-	if (status != PJ_STATUS_FROM_OS(EADDRINUSE))
-	    break;
-	PJ_LOG(4,(THIS_FILE, "Address is still in use, retrying.."));
-	pj_thread_sleep(msec);
-    }
-
-    if (status != PJ_SUCCESS)
-	goto on_error;
-
-    if (flags & HAS_QOS) {
-	status = pj_sock_set_qos_params(new_sock, &qos_params);
-	if (status != PJ_SUCCESS)
-	    goto on_error;
-    }
-
-    if (flags & HAS_PEER_ADDR) {
-	status = pj_sock_connect(new_sock, &rem_addr, addr_len);
-	if (status != PJ_SUCCESS)
-	    goto on_error;
-    }
-
+        goto on_error;
+    
     /* Set socket to nonblocking. */
     val = 1;
 #if defined(PJ_WIN32) && PJ_WIN32!=0 || \
@@ -771,24 +869,17 @@ static pj_status_t replace_udp_sock(pj_ioqueue_key_t *h)
     if (ioctl(new_sock, FIONBIO, &val)) {
 #endif
         status = pj_get_netos_error();
-	goto on_error;
+        goto on_error;
     }
 
     /* Replace the occurrence of old socket with new socket in the
      * fd sets.
      */
-    fds_cnt = 0;
-    fds[fds_cnt++] = &h->ioqueue->rfdset;
-    fds[fds_cnt++] = &h->ioqueue->wfdset;
-#if PJ_HAS_TCP
-    fds[fds_cnt++] = &h->ioqueue->xfdset;
-#endif
-
     for (i=0; i<fds_cnt; ++i) {
-	if (PJ_FD_ISSET(old_sock, fds[i])) {
-	    PJ_FD_CLR(old_sock, fds[i]);
-	    PJ_FD_SET(new_sock, fds[i]);
-	}
+        if (PJ_FD_ISSET(h->fd, fds[i])) {
+            PJ_FD_CLR(h->fd, fds[i]);
+            PJ_FD_SET(new_sock, fds[i]);
+        }
     }
 
     /* And finally replace the fd in the key */
@@ -802,10 +893,21 @@ static pj_status_t replace_udp_sock(pj_ioqueue_key_t *h)
 
 on_error:
     if (new_sock != PJ_INVALID_SOCKET)
-	pj_sock_close(new_sock);
-    PJ_PERROR(1,(THIS_FILE, status, "Error replacing socket"));
+        pj_sock_close(new_sock);
+    if (old_sock != PJ_INVALID_SOCKET)
+        pj_sock_close(old_sock);
+
+    /* Clear the occurrence of old socket in the fd sets. */
+    for (i=0; i<fds_cnt; ++i) {
+        if (PJ_FD_ISSET(h->fd, fds[i])) {
+            PJ_FD_CLR(h->fd, fds[i]);
+        }
+    }
+
+    h->fd = PJ_INVALID_SOCKET;
+    PJ_PERROR(1,(THIS_FILE, status, "Error replacing socket %ld", old_sock));
     pj_lock_release(h->ioqueue->lock);
-    return status;
+    return PJ_ESOCKETSTOP;
 }
 #endif
 
@@ -831,16 +933,23 @@ PJ_DEF(int) pj_ioqueue_poll( pj_ioqueue_t *ioqueue, const pj_time_val *timeout)
 {
     pj_fd_set_t rfdset, wfdset, xfdset;
     int nfds;
-    int count, i, counter;
+    int i, count, event_cnt, processed_cnt;
     pj_ioqueue_key_t *h;
+    enum { MAX_EVENTS = PJ_IOQUEUE_MAX_CAND_EVENTS };
     struct event
     {
-        pj_ioqueue_key_t	*key;
+        pj_ioqueue_key_t        *key;
         enum ioqueue_event_type  event_type;
-    } event[PJ_IOQUEUE_MAX_EVENTS_IN_SINGLE_POLL];
+    } event[MAX_EVENTS];
 
     PJ_ASSERT_RETURN(ioqueue, -PJ_EINVAL);
 
+#if defined(PJ_HAS_SSL_SOCK) && PJ_HAS_SSL_SOCK != 0 && \
+    (PJ_SSL_SOCK_IMP == PJ_SSL_SOCK_IMP_APPLE)
+    /* Call SSL Network framework event poll */
+    ssl_network_event_poll();
+#endif
+
     /* Lock ioqueue before making fd_set copies */
     pj_lock_acquire(ioqueue->lock);
 
@@ -852,13 +961,13 @@ PJ_DEF(int) pj_ioqueue_poll( pj_ioqueue_t *ioqueue, const pj_time_val *timeout)
 #if defined(PJ_HAS_TCP) && PJ_HAS_TCP!=0
         && PJ_FD_COUNT(&ioqueue->xfdset)==0
 #endif
-	)
+        )
     {
 #if PJ_IOQUEUE_HAS_SAFE_UNREG
-	scan_closing_keys(ioqueue);
+        scan_closing_keys(ioqueue);
 #endif
-	pj_lock_release(ioqueue->lock);
-	TRACE__((THIS_FILE, "     poll: no fd is set"));
+        pj_lock_release(ioqueue->lock);
+        TRACE__((THIS_FILE, "     poll: no fd is set"));
         if (timeout)
             pj_thread_sleep(PJ_TIME_VAL_MSEC(*timeout));
         return 0;
@@ -882,15 +991,26 @@ PJ_DEF(int) pj_ioqueue_poll( pj_ioqueue_t *ioqueue, const pj_time_val *timeout)
     /* Unlock ioqueue before select(). */
     pj_lock_release(ioqueue->lock);
 
+#if defined(PJ_WIN32_WINPHONE8) && PJ_WIN32_WINPHONE8
+    count = 0;
+    __try {
+#endif
+
     count = pj_sock_select(nfds+1, &rfdset, &wfdset, &xfdset, 
-			   timeout);
+                           timeout);
+
+#if defined(PJ_WIN32_WINPHONE8) && PJ_WIN32_WINPHONE8
+    /* Ignore Invalid Handle Exception raised by select().*/
+    }
+    __except (GetExceptionCode() == STATUS_INVALID_HANDLE ?
+              EXCEPTION_CONTINUE_EXECUTION : EXCEPTION_CONTINUE_SEARCH) {
+    }
+#endif    
     
     if (count == 0)
-	return 0;
+        return 0;
     else if (count < 0)
-	return -pj_get_netos_error();
-    else if (count > PJ_IOQUEUE_MAX_EVENTS_IN_SINGLE_POLL)
-        count = PJ_IOQUEUE_MAX_EVENTS_IN_SINGLE_POLL;
+        return -pj_get_netos_error();
 
     /* Scan descriptor sets for event and add the events in the event
      * array to be processed later in this function. We do this so that
@@ -898,55 +1018,59 @@ PJ_DEF(int) pj_ioqueue_poll( pj_ioqueue_t *ioqueue, const pj_time_val *timeout)
      */
     pj_lock_acquire(ioqueue->lock);
 
-    counter = 0;
+    event_cnt = 0;
 
     /* Scan for writable sockets first to handle piggy-back data
      * coming with accept().
      */
-    h = ioqueue->active_list.next;
-    for ( ; h!=&ioqueue->active_list && counter<count; h = h->next) {
+    for (h = ioqueue->active_list.next;
+         h != &ioqueue->active_list && event_cnt < MAX_EVENTS;
+         h = h->next)
+    {
+        if (h->fd == PJ_INVALID_SOCKET)
+            continue;
 
-	if ( (key_has_pending_write(h) || key_has_pending_connect(h))
-	     && PJ_FD_ISSET(h->fd, &wfdset) && !IS_CLOSING(h))
+        if ( (key_has_pending_write(h) || key_has_pending_connect(h))
+             && PJ_FD_ISSET(h->fd, &wfdset) && !IS_CLOSING(h))
         {
 #if PJ_IOQUEUE_HAS_SAFE_UNREG
-	    increment_counter(h);
+            increment_counter(h);
 #endif
-            event[counter].key = h;
-            event[counter].event_type = WRITEABLE_EVENT;
-            ++counter;
+            event[event_cnt].key = h;
+            event[event_cnt].event_type = WRITEABLE_EVENT;
+            ++event_cnt;
         }
 
         /* Scan for readable socket. */
-	if ((key_has_pending_read(h) || key_has_pending_accept(h))
+        if ((key_has_pending_read(h) || key_has_pending_accept(h))
             && PJ_FD_ISSET(h->fd, &rfdset) && !IS_CLOSING(h) &&
-	    counter<count)
+            event_cnt < MAX_EVENTS)
         {
 #if PJ_IOQUEUE_HAS_SAFE_UNREG
-	    increment_counter(h);
+            increment_counter(h);
 #endif
-            event[counter].key = h;
-            event[counter].event_type = READABLE_EVENT;
-            ++counter;
-	}
+            event[event_cnt].key = h;
+            event[event_cnt].event_type = READABLE_EVENT;
+            ++event_cnt;
+        }
 
 #if PJ_HAS_TCP
         if (key_has_pending_connect(h) && PJ_FD_ISSET(h->fd, &xfdset) &&
-	    !IS_CLOSING(h) && counter<count) 
-	{
+            !IS_CLOSING(h) && event_cnt < MAX_EVENTS)
+        {
 #if PJ_IOQUEUE_HAS_SAFE_UNREG
-	    increment_counter(h);
+            increment_counter(h);
 #endif
-            event[counter].key = h;
-            event[counter].event_type = EXCEPTION_EVENT;
-            ++counter;
+            event[event_cnt].key = h;
+            event[event_cnt].event_type = EXCEPTION_EVENT;
+            ++event_cnt;
         }
 #endif
     }
 
-    for (i=0; i<counter; ++i) {
-	if (event[i].key->grp_lock)
-	    pj_grp_lock_add_ref_dbg(event[i].key->grp_lock, "ioqueue", 0);
+    for (i=0; i<event_cnt; ++i) {
+        if (event[i].key->grp_lock)
+            pj_grp_lock_add_ref_dbg(event[i].key->grp_lock, "ioqueue", 0);
     }
 
     PJ_RACE_ME(5);
@@ -955,37 +1079,53 @@ PJ_DEF(int) pj_ioqueue_poll( pj_ioqueue_t *ioqueue, const pj_time_val *timeout)
 
     PJ_RACE_ME(5);
 
-    count = counter;
+    processed_cnt = 0;
 
     /* Now process all events. The dispatch functions will take care
      * of locking in each of the key
      */
-    for (counter=0; counter<count; ++counter) {
-        switch (event[counter].event_type) {
-        case READABLE_EVENT:
-            ioqueue_dispatch_read_event(ioqueue, event[counter].key);
-            break;
-        case WRITEABLE_EVENT:
-            ioqueue_dispatch_write_event(ioqueue, event[counter].key);
-            break;
-        case EXCEPTION_EVENT:
-            ioqueue_dispatch_exception_event(ioqueue, event[counter].key);
-            break;
-        case NO_EVENT:
-            pj_assert(!"Invalid event!");
-            break;
+    for (i=0; i<event_cnt; ++i) {
+
+        /* Just do not exceed PJ_IOQUEUE_MAX_EVENTS_IN_SINGLE_POLL */
+        if (processed_cnt < PJ_IOQUEUE_MAX_EVENTS_IN_SINGLE_POLL) {
+            switch (event[i].event_type) {
+            case READABLE_EVENT:
+                if (ioqueue_dispatch_read_event(ioqueue, event[i].key))
+                    ++processed_cnt;
+                break;
+            case WRITEABLE_EVENT:
+                if (ioqueue_dispatch_write_event(ioqueue, event[i].key))
+                    ++processed_cnt;
+                break;
+            case EXCEPTION_EVENT:
+                if (ioqueue_dispatch_exception_event(ioqueue, event[i].key))
+                    ++processed_cnt;
+                break;
+            case NO_EVENT:
+                pj_assert(!"Invalid event!");
+                break;
+            }
         }
 
 #if PJ_IOQUEUE_HAS_SAFE_UNREG
-	decrement_counter(event[counter].key);
+        decrement_counter(event[i].key);
 #endif
 
-	if (event[counter].key->grp_lock)
-	    pj_grp_lock_dec_ref_dbg(event[counter].key->grp_lock,
-	                            "ioqueue", 0);
+        if (event[i].key->grp_lock)
+            pj_grp_lock_dec_ref_dbg(event[i].key->grp_lock,
+                                    "ioqueue", 0);
     }
 
+    TRACE__((THIS_FILE, "     poll: count=%d events=%d processed=%d",
+             count, event_cnt, processed_cnt));
 
-    return count;
+    return processed_cnt;
 }
 
+PJ_DEF(pj_oshandle_t) pj_ioqueue_get_os_handle( pj_ioqueue_t *ioqueue )
+{
+    PJ_UNUSED_ARG(ioqueue);
+    return NULL;
+}
+
+#endif /* PJ_IOQUEUE_IMP == PJ_IOQUEUE_IMP_SELECT */
