@@ -1,4 +1,3 @@
-/* $Id$ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -28,19 +27,25 @@
 
 PJ_DEF(pj_status_t) pj_time_decode(const pj_time_val *tv, pj_parsed_time *pt)
 {
-    struct tm *local_time;
+    struct tm local_time;
+    time_t sec = tv->sec; // time_t might be 64 bit even when long int is 32 bit, (time_t*) dangerous
 
     PJ_CHECK_STACK();
 
-    local_time = localtime((time_t*)&tv->sec);
+#if defined(PJ_HAS_LOCALTIME_R) && PJ_HAS_LOCALTIME_R != 0
+    localtime_r(&sec, &local_time);
+#else
+    /* localtime() is NOT thread-safe. */
+    local_time = *localtime(&sec);
+#endif
 
-    pt->year = local_time->tm_year+1900;
-    pt->mon = local_time->tm_mon;
-    pt->day = local_time->tm_mday;
-    pt->hour = local_time->tm_hour;
-    pt->min = local_time->tm_min;
-    pt->sec = local_time->tm_sec;
-    pt->wday = local_time->tm_wday;
+    pt->year = local_time.tm_year+1900;
+    pt->mon = local_time.tm_mon;
+    pt->day = local_time.tm_mday;
+    pt->hour = local_time.tm_hour;
+    pt->min = local_time.tm_min;
+    pt->sec = local_time.tm_sec;
+    pt->wday = local_time.tm_wday;
     pt->msec = tv->msec;
 
     return PJ_SUCCESS;
@@ -67,16 +72,31 @@ PJ_DEF(pj_status_t) pj_time_encode(const pj_parsed_time *pt, pj_time_val *tv)
     return PJ_SUCCESS;
 }
 
-#endif /* !PJ_WIN32 */
+static int get_tz_offset_secs()
+{
+    time_t epoch_plus_11h = 60 * 60 * 11;
+    struct tm ltime, gtime;
+    int offset_min;
 
+#if defined(PJ_HAS_LOCALTIME_R) && PJ_HAS_LOCALTIME_R != 0
+    localtime_r(&epoch_plus_11h, &ltime);
+    gmtime_r(&epoch_plus_11h, &gtime);
+#else
+    ltime = *localtime(&epoch_plus_11h);
+    gtime = *gmtime(&epoch_plus_11h);
+#endif
+
+    offset_min = (ltime.tm_hour*60+ltime.tm_min) - (gtime.tm_hour*60+gtime.tm_min);
+    return offset_min*60;
+}
 
 /**
  * Convert local time to GMT.
  */
 PJ_DEF(pj_status_t) pj_time_local_to_gmt(pj_time_val *tv)
 {
-    PJ_UNUSED_ARG(tv);
-    return PJ_EBUG;
+    tv->sec -= get_tz_offset_secs();
+    return PJ_SUCCESS;
 }
 
 /**
@@ -84,8 +104,8 @@ PJ_DEF(pj_status_t) pj_time_local_to_gmt(pj_time_val *tv)
  */
 PJ_DEF(pj_status_t) pj_time_gmt_to_local(pj_time_val *tv)
 {
-    PJ_UNUSED_ARG(tv);
-    return PJ_EBUG;
+    tv->sec += get_tz_offset_secs();
+    return PJ_SUCCESS;
 }
 
-
+#endif /* !PJ_WIN32 */
