@@ -1,4 +1,3 @@
-/* $Id$ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -56,9 +55,9 @@ static int get_machine_speed_mhz()
     char buf[512];
     int len;
     char *pos, *end;
-	
+        
     PJ_CHECK_STACK();
-	
+        
     /* Open /proc/cpuinfo and read the file */
     strm = fopen("/proc/cpuinfo", "r");
     if (!strm)
@@ -88,15 +87,15 @@ static int get_machine_speed_mhz()
 PJ_DEF(pj_status_t) pj_get_timestamp(pj_timestamp *ts)
 {
     if (machine_speed_mhz == 0) {
-	machine_speed_mhz = get_machine_speed_mhz();
-	if (machine_speed_mhz > 0) {
-	    machine_speed.u64 = machine_speed_mhz * 1000000.0;
-	}
+        machine_speed_mhz = get_machine_speed_mhz();
+        if (machine_speed_mhz > 0) {
+            machine_speed.u64 = machine_speed_mhz * 1000000.0;
+        }
     }
     
     if (machine_speed_mhz == -1) {
-	ts->u64 = 0;
-	return -1;
+        ts->u64 = 0;
+        return -1;
     } 
     ts->u64 = rdtsc();
     return 0;
@@ -105,15 +104,15 @@ PJ_DEF(pj_status_t) pj_get_timestamp(pj_timestamp *ts)
 PJ_DEF(pj_status_t) pj_get_timestamp_freq(pj_timestamp *freq)
 {
     if (machine_speed_mhz == 0) {
-	machine_speed_mhz = get_machine_speed_mhz();
-	if (machine_speed_mhz > 0) {
-	    machine_speed.u64 = machine_speed_mhz * 1000000.0;
-	}
+        machine_speed_mhz = get_machine_speed_mhz();
+        if (machine_speed_mhz > 0) {
+            machine_speed.u64 = machine_speed_mhz * 1000000.0;
+        }
     }
     
     if (machine_speed_mhz == -1) {
-	freq->u64 = 1;	/* return 1 to prevent division by zero in apps. */
-	return -1;
+        freq->u64 = 1;  /* return 1 to prevent division by zero in apps. */
+        return -1;
     } 
 
     freq->u64 = machine_speed.u64;
@@ -121,28 +120,86 @@ PJ_DEF(pj_status_t) pj_get_timestamp_freq(pj_timestamp *freq)
 }
 
 #elif defined(PJ_DARWINOS) && PJ_DARWINOS != 0
-#include <mach/mach.h>
-#include <mach/clock.h>
-#include <errno.h>
 
-#ifndef NSEC_PER_SEC
-#	define NSEC_PER_SEC	1000000000
+#include <mach/mach.h>
+#include <mach/mach_time.h>
+
+static mach_timebase_info_data_t timebase;
+
+PJ_DEF(pj_status_t) pj_get_timestamp(pj_timestamp *ts)
+{
+    if (!timebase.denom) {
+        mach_timebase_info(&timebase);
+    }
+
+    ts->u64 = mach_continuous_time() * timebase.numer / timebase.denom;
+
+    return PJ_SUCCESS;
+}
+
+PJ_DEF(pj_status_t) pj_get_timestamp_freq(pj_timestamp *freq)
+{
+    freq->u32.hi = 0;
+    freq->u32.lo = NSEC_PER_SEC;
+
+    return PJ_SUCCESS;
+}
+
+#elif defined(__ANDROID__)
+
+#include <errno.h>
+#include <time.h>
+
+#if defined(PJ_HAS_ANDROID_ALARM_H) && PJ_HAS_ANDROID_ALARM_H != 0
+#  include <linux/android_alarm.h>
+#  include <fcntl.h>
+#endif
+
+#define NSEC_PER_SEC    1000000000
+
+#if defined(ANDROID_ALARM_GET_TIME)
+static int s_alarm_fd = -1;
+
+void close_alarm_fd()
+{
+    if (s_alarm_fd != -1)
+        close(s_alarm_fd);
+    s_alarm_fd = -1;
+}
 #endif
 
 PJ_DEF(pj_status_t) pj_get_timestamp(pj_timestamp *ts)
 {
-    mach_timespec_t tp;
-    int ret;
-    clock_serv_t serv;
+    struct timespec tp;
+    int err = -1;
 
-    ret = host_get_clock_service(mach_host_self(), SYSTEM_CLOCK, &serv);
-    if (ret != KERN_SUCCESS) {
-	return PJ_RETURN_OS_ERROR(EINVAL);
+#if defined(ANDROID_ALARM_GET_TIME)
+    if (s_alarm_fd == -1) {
+        int fd = open("/dev/alarm", O_RDONLY);
+        if (fd >= 0) {
+            s_alarm_fd = fd;
+            pj_atexit(&close_alarm_fd);
+        }
+    }
+    
+    if (s_alarm_fd != -1) {
+        err = ioctl(s_alarm_fd,
+              ANDROID_ALARM_GET_TIME(ANDROID_ALARM_ELAPSED_REALTIME), &tp);
+    }
+#elif defined(CLOCK_BOOTTIME)
+    err = clock_gettime(CLOCK_BOOTTIME, &tp);
+#endif
+    
+    if (err != 0) {
+        /* Fallback to CLOCK_MONOTONIC if /dev/alarm is not found, or
+         * getting ANDROID_ALARM_ELAPSED_REALTIME fails, or 
+         * CLOCK_BOOTTIME fails.
+         */
+        err = clock_gettime(CLOCK_MONOTONIC, &tp);
     }
 
-    ret = clock_get_time(serv, &tp);
-    if (ret != KERN_SUCCESS) {
-	return PJ_RETURN_OS_ERROR(EINVAL);
+    if (err != 0) {
+        return PJ_RETURN_OS_ERROR(pj_get_native_os_error());
     }
 
     ts->u64 = tp.tv_sec;
@@ -165,14 +222,14 @@ PJ_DEF(pj_status_t) pj_get_timestamp_freq(pj_timestamp *freq)
 #include <time.h>
 #include <errno.h>
 
-#define NSEC_PER_SEC	1000000000
+#define NSEC_PER_SEC    1000000000
 
 PJ_DEF(pj_status_t) pj_get_timestamp(pj_timestamp *ts)
 {
     struct timespec tp;
 
     if (clock_gettime(CLOCK_MONOTONIC, &tp) != 0) {
-	return PJ_RETURN_OS_ERROR(pj_get_native_os_error());
+        return PJ_RETURN_OS_ERROR(pj_get_native_os_error());
     }
 
     ts->u64 = tp.tv_sec;
@@ -194,14 +251,14 @@ PJ_DEF(pj_status_t) pj_get_timestamp_freq(pj_timestamp *freq)
 #include <sys/time.h>
 #include <errno.h>
 
-#define USEC_PER_SEC	1000000
+#define USEC_PER_SEC    1000000
 
 PJ_DEF(pj_status_t) pj_get_timestamp(pj_timestamp *ts)
 {
     struct timeval tv;
 
     if (gettimeofday(&tv, NULL) != 0) {
-	return PJ_RETURN_OS_ERROR(pj_get_native_os_error());
+        return PJ_RETURN_OS_ERROR(pj_get_native_os_error());
     }
 
     ts->u64 = tv.tv_sec;
