@@ -36,7 +36,7 @@
 #if defined(PJ_HAS_SSL_SOCK) && PJ_HAS_SSL_SOCK != 0 && \
     (PJ_SSL_SOCK_IMP == PJ_SSL_SOCK_IMP_OPENSSL)
 
-#include "ssl_sock_imp_common.c"
+#include "ssl_sock_imp_common.h"
 
 #define THIS_FILE               "ssl_sock_ossl.c"
 
@@ -159,21 +159,34 @@ static void update_certs_info(pj_ssl_sock_t* ssock,
                               pj_ssl_cert_info *remote_cert_info,
                               pj_bool_t is_verify);
 
-#if !USING_LIBRESSL && OPENSSL_VERSION_NUMBER >= 0x10100000L
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
 #  define OPENSSL_NO_SSL2           /* seems to be removed in 1.1.0 */
+# ifndef M_ASN1_STRING_data
 #  define M_ASN1_STRING_data(x)     ASN1_STRING_get0_data(x)
 #  define M_ASN1_STRING_length(x)   ASN1_STRING_length(x)
-#  if defined(OPENSSL_API_COMPAT) && OPENSSL_API_COMPAT >= 0x10100000L
+# endif
+#  if defined(OPENSSL_API_COMPAT) && OPENSSL_API_COMPAT >= 0x10100000L || \
+      defined(OPENSSL_NO_DEPRECATED)
+
 #     define X509_get_notBefore(x)  X509_get0_notBefore(x)
 #     define X509_get_notAfter(x)   X509_get0_notAfter(x)
+
+#    if OPENSSL_VERSION_NUMBER >= 0x30000000L
+#      if defined(OPENSSL_API_COMPAT) && OPENSSL_API_COMPAT >= 0x30000000L || \
+          defined(OPENSSL_NO_DEPRECATED)
+
+#         define SSL_get_peer_certificate(x)     SSL_get1_peer_certificate(x)
+
+#      endif
+#    endif
+
 #  endif
-#elif !USING_LIBRESSL
+#else
 #  define SSL_CIPHER_get_id(c)      (c)->id
 #  define SSL_set_session(ssl, s)   (ssl)->session = (s)
 #  define X509_STORE_CTX_get0_cert(ctx) ((ctx)->cert)
 #endif
 
-
 #ifdef _MSC_VER
 #  if OPENSSL_VERSION_NUMBER >= 0x10100000L
 #    pragma comment(lib, "libcrypto")
@@ -191,7 +204,7 @@ static void update_certs_info(pj_ssl_sock_t* ssock,
 #  ifdef _MSC_VER
 #    define strerror_r(err,buf,len) strerror_s(buf,len,err)
 #  else
-#    define strerror_r(err,buf,len) pj_ansi_strncpy(buf,strerror(err),len)
+#    define strerror_r(err,buf,len) pj_ansi_strxcpy(buf,strerror(err),len)
 #  endif
 #endif
 
@@ -219,6 +232,10 @@ typedef struct ossl_sock_t
     BIO                  *ossl_wbio;
 } ossl_sock_t;
 
+
+#include "ssl_sock_imp_common.c"
+
+
 /**
  * Mapping from OpenSSL error codes to pjlib error space.
  */
@@ -345,7 +362,7 @@ static void SSLLogErrors(char * action, int ret, int ssl_err, int len,
         break;
     }
     default:
-        PJ_LOG(2,("SSL", "%lu [%s] (%s) ret: %d len: %d",
+        PJ_LOG(2,("SSL", "%d [%s] (%s) ret: %d len: %d",
                   ssl_err, ssl_err_str, action, ret, len));
         break;
     }
@@ -441,7 +458,7 @@ static pj_str_t ssl_strerror(pj_status_t status,
         const char *tmp = NULL;
         tmp = ERR_reason_error_string(ssl_err);
         if (tmp) {
-            pj_ansi_strncpy(buf, tmp, bufsize);
+            pj_ansi_strxcpy(buf, tmp, bufsize);
             errstr = pj_str(buf);
             return errstr;
         }
@@ -466,11 +483,12 @@ static pj_str_t ssl_strerror(pj_status_t status,
 */
 static const struct ssl_ciphers_t ADDITIONAL_CIPHERS[] = {
         {0xFF000000, "DEFAULT"},
-        {0xFF000001, "@SECLEVEL=1"},
-        {0xFF000002, "@SECLEVEL=2"},
-        {0xFF000003, "@SECLEVEL=3"},
-        {0xFF000004, "@SECLEVEL=4"},
-        {0xFF000005, "@SECLEVEL=5"}
+        {0xFF000001, "@SECLEVEL=0"},
+        {0xFF000002, "@SECLEVEL=1"},
+        {0xFF000003, "@SECLEVEL=2"},
+        {0xFF000004, "@SECLEVEL=3"},
+        {0xFF000005, "@SECLEVEL=4"},
+        {0xFF000006, "@SECLEVEL=5"}
 };
 static const unsigned int ADDITIONAL_CIPHER_COUNT = 
     sizeof (ADDITIONAL_CIPHERS) / sizeof (ADDITIONAL_CIPHERS[0]);
@@ -678,7 +696,7 @@ static pj_status_t init_openssl(void)
 
     openssl_init_count = 1;
 
-    PJ_LOG(4, (THIS_FILE, "OpenSSL version : %x", OPENSSL_VERSION_NUMBER));
+    PJ_LOG(4, (THIS_FILE, "OpenSSL version : %ld", OPENSSL_VERSION_NUMBER));
     /* Register error subsystem */
     status = pj_register_strerror(PJ_SSL_ERRNO_START, 
                                   PJ_SSL_ERRNO_SPACE_SIZE, 
@@ -695,6 +713,7 @@ static pj_status_t init_openssl(void)
 #if OPENSSL_VERSION_NUMBER < 0x009080ffL
     /* This is now synonym of SSL_library_init() */
     OpenSSL_add_all_algorithms();
+    OpenSSL_add_all_digests();
 #endif
 
     /* Init available ciphers */
@@ -766,7 +785,7 @@ static pj_status_t init_openssl(void)
 #if !USING_LIBRESSL && !defined(OPENSSL_NO_EC) \
     && OPENSSL_VERSION_NUMBER >= 0x1000200fL
 #if OPENSSL_VERSION_NUMBER >= 0x1010100fL
-        ssl_curves_num = EC_get_builtin_curves(NULL, 0);
+        ssl_curves_num = (unsigned)EC_get_builtin_curves(NULL, 0);
 #else
 
 #if USING_BORINGSSL
@@ -935,15 +954,6 @@ static int verify_cb(int preverify_ok, X509_STORE_CTX *x509_ctx)
         goto on_return;
     }
 
-    if (ssock->param.cb.on_verify_cb) {
-        update_certs_info(ssock, x509_ctx, &ssock->local_cert_info, 
-                          &ssock->remote_cert_info, PJ_TRUE);
-        preverify_ok = (*ssock->param.cb.on_verify_cb)(ssock, 
-                                                       ssock->is_server);
-
-        goto on_return;
-    }
-
     /* Store verification status */
     err = X509_STORE_CTX_get_error(x509_ctx);
     switch (err) {
@@ -1014,6 +1024,16 @@ static int verify_cb(int preverify_ok, X509_STORE_CTX *x509_ctx)
         break;
     }
 
+    /* Invoke app's verification callback */
+    if (ssock->param.cb.on_verify_cb) {
+        update_certs_info(ssock, x509_ctx, &ssock->local_cert_info,
+                          &ssock->remote_cert_info, PJ_TRUE);
+        preverify_ok = (*ssock->param.cb.on_verify_cb)(ssock,
+                                                       ssock->is_server);
+
+        goto on_return;
+    }
+
     /* When verification is not requested just return ok here, however
      * application can still get the verification status.
      */
@@ -1056,7 +1076,13 @@ static int xname_cmp(const X509_NAME **a, const X509_NAME **b) {
 #if !defined(OPENSSL_NO_DH)
 
 static void set_option(const pj_ssl_sock_t* ssock, SSL_CTX* ctx) {
-    unsigned long options = SSL_OP_CIPHER_SERVER_PREFERENCE |
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L && PJ_HAS_INT64
+    uint64_t options;
+#else
+    unsigned long options;
+#endif
+
+    options = SSL_OP_CIPHER_SERVER_PREFERENCE |
 #if !defined(OPENSSL_NO_ECDH) && OPENSSL_VERSION_NUMBER >= 0x10000000L
         SSL_OP_SINGLE_ECDH_USE |
 #endif
@@ -1117,37 +1143,13 @@ static pj_status_t init_ossl_ctx(pj_ssl_sock_t *ssock)
     int rc;
     pj_status_t status;
 
-    if (ssock->param.proto == PJ_SSL_SOCK_PROTO_DEFAULT)
-        ssock->param.proto = PJ_SSL_SOCK_PROTO_SSL23;
-
-    /* Determine SSL method to use */
-    /* Specific version methods are deprecated since 1.1.0 */
-#if (USING_LIBRESSL && LIBRESSL_VERSION_NUMBER < 0x2020100fL)\
-    || OPENSSL_VERSION_NUMBER < 0x10100000L
-    switch (ssock->param.proto) {
-    case PJ_SSL_SOCK_PROTO_TLS1:
-        ssl_method = (SSL_METHOD*)TLSv1_method();
-        break;
-#ifndef OPENSSL_NO_SSL2
-    case PJ_SSL_SOCK_PROTO_SSL2:
-        ssl_method = (SSL_METHOD*)SSLv2_method();
-        break;
-#endif
-#ifndef OPENSSL_NO_SSL3_METHOD
-    case PJ_SSL_SOCK_PROTO_SSL3:
-        ssl_method = (SSL_METHOD*)SSLv3_method();
-#endif
-        break;
+    if (ssock->param.proto == PJ_SSL_SOCK_PROTO_DEFAULT) {
+        ssock->param.proto = PJ_SSL_SOCK_PROTO_TLS1_2 |
+                             PJ_SSL_SOCK_PROTO_TLS1_3;
     }
-#endif
 
     if (!ssl_method) {
-#if (USING_LIBRESSL && LIBRESSL_VERSION_NUMBER < 0x2020100fL)\
-    || OPENSSL_VERSION_NUMBER < 0x10100000L
-        ssl_method = (SSL_METHOD*)SSLv23_method();
-#else
         ssl_method = (SSL_METHOD*)TLS_method();
-#endif
 
 #ifdef SSL_OP_NO_SSLv2
         /** Check if SSLv2 is enabled */
@@ -1222,6 +1224,12 @@ static pj_status_t init_ossl_ctx(pj_ssl_sock_t *ssock)
         }
     }
 
+#ifdef SSL_OP_NO_RENEGOTIATION
+    if (!ssock->param.enable_renegotiation) {
+        ssl_opt |= SSL_OP_NO_RENEGOTIATION;
+    }
+#endif
+
     if (ssl_opt)
         SSL_CTX_set_options(ctx, ssl_opt);
 
@@ -1235,6 +1243,10 @@ static pj_status_t init_ossl_ctx(pj_ssl_sock_t *ssock)
 
     /* Apply credentials */
     if (cert) {
+        pj_bool_t CA_loaded = PJ_FALSE;
+        pj_bool_t cert_loaded = PJ_FALSE;
+        pj_bool_t privkey_loaded = PJ_FALSE;
+
         /* Load CA list if one is specified. */
         if (cert->CA_file.slen || cert->CA_path.slen) {
 
@@ -1259,6 +1271,7 @@ static pj_status_t init_ossl_ctx(pj_ssl_sock_t *ssock)
                 ossock->ossl_ctx = NULL;
                 return status;
             } else {
+                CA_loaded = PJ_TRUE;
                 PJ_LOG(4,(ssock->pool->obj_name,
                           "CA certificates loaded from '%s%s%s'",
                           cert->CA_file.ptr,
@@ -1290,6 +1303,7 @@ static pj_status_t init_ossl_ctx(pj_ssl_sock_t *ssock)
                 ossock->ossl_ctx = NULL;
                 return status;
             } else {
+                cert_loaded = PJ_TRUE;
                 PJ_LOG(4,(ssock->pool->obj_name,
                           "Certificate chain loaded from '%s'",
                           cert->cert_file.ptr));
@@ -1312,6 +1326,7 @@ static pj_status_t init_ossl_ctx(pj_ssl_sock_t *ssock)
                 ossock->ossl_ctx = NULL;
                 return status;
             } else {
+                privkey_loaded = PJ_TRUE;
                 PJ_LOG(4,(ssock->pool->obj_name,
                           "Private key loaded from '%s'",
                           cert->privkey_file.ptr));
@@ -1329,12 +1344,12 @@ static pj_status_t init_ossl_ctx(pj_ssl_sock_t *ssock)
         }
 
         /* Load from buffer. */
-        if (cert->cert_buf.slen) {
+        if (cert->cert_buf.slen && !cert_loaded) {
             BIO *cbio;
             X509 *xcert = NULL;
             
             cbio = BIO_new_mem_buf((void*)cert->cert_buf.ptr,
-                                   cert->cert_buf.slen);
+                                   (int)cert->cert_buf.slen);
             if (cbio != NULL) {
                 xcert = PEM_read_bio_X509(cbio, NULL, 0, NULL);
                 if (xcert != NULL) {
@@ -1349,6 +1364,7 @@ static pj_status_t init_ossl_ctx(pj_ssl_sock_t *ssock)
                         ossock->ossl_ctx = NULL;
                         return status;
                     } else {
+                        cert_loaded = PJ_TRUE;
                         PJ_LOG(4,(ssock->pool->obj_name,
                                   "Certificate chain loaded from buffer"));
                     }
@@ -1358,9 +1374,9 @@ static pj_status_t init_ossl_ctx(pj_ssl_sock_t *ssock)
             }       
         }
 
-        if (cert->CA_buf.slen) {
+        if (cert->CA_buf.slen && !CA_loaded) {
             BIO *cbio = BIO_new_mem_buf((void*)cert->CA_buf.ptr,
-                                        cert->CA_buf.slen);
+                                        (int)cert->CA_buf.slen);
             X509_STORE *cts = SSL_CTX_get_cert_store(ctx);
 
             if (cbio && cts) {
@@ -1388,6 +1404,7 @@ static pj_status_t init_ossl_ctx(pj_ssl_sock_t *ssock)
 #endif
                         }
                     }
+                    CA_loaded = (cnt > 0);
                     PJ_LOG(4,(ssock->pool->obj_name,
                               "CA certificates loaded from buffer (cnt=%d)",
                               cnt));
@@ -1397,12 +1414,12 @@ static pj_status_t init_ossl_ctx(pj_ssl_sock_t *ssock)
             }
         }
 
-        if (cert->privkey_buf.slen) {
+        if (cert->privkey_buf.slen && !privkey_loaded) {
             BIO *kbio;      
             EVP_PKEY *pkey = NULL;
 
             kbio = BIO_new_mem_buf((void*)cert->privkey_buf.ptr,
-                                   cert->privkey_buf.slen);
+                                   (int)cert->privkey_buf.slen);
             if (kbio != NULL) {
                 pkey = PEM_read_bio_PrivateKey(kbio, NULL, &password_cb,
                                                cert);
@@ -1418,6 +1435,7 @@ static pj_status_t init_ossl_ctx(pj_ssl_sock_t *ssock)
                         ossock->ossl_ctx = NULL;
                         return status;
                     } else {
+                        privkey_loaded = PJ_TRUE;
                         PJ_LOG(4,(ssock->pool->obj_name,
                                   "Private key loaded from buffer"));
                     }
@@ -1433,8 +1451,45 @@ static pj_status_t init_ossl_ctx(pj_ssl_sock_t *ssock)
                 BIO_free(kbio);
             }       
         }
+
+        if ((cert->direct.type & PJ_SSL_CERT_DIRECT_OPENSSL_EVP_PKEY) &&
+             cert->direct.privkey && !privkey_loaded)
+        {
+            EVP_PKEY *pkey = (EVP_PKEY*)cert->direct.privkey;
+            rc = SSL_CTX_use_PrivateKey(ctx, pkey);
+            if (rc != 1) {
+                status = GET_SSL_STATUS(ssock);
+                PJ_PERROR(1,(ssock->pool->obj_name, status,
+                             "Error setting direct private key"));
+                SSL_CTX_free(ctx);
+                ossock->ossl_ctx = NULL;
+                return status;
+            } else {
+                privkey_loaded = PJ_TRUE;
+                PJ_LOG(4,(ssock->pool->obj_name, "Direct private key set"));
+            }
+        }
+
+        if ((cert->direct.type & PJ_SSL_CERT_DIRECT_OPENSSL_X509_CERT) &&
+             cert->direct.cert && !cert_loaded)
+        {
+            X509 *xcert = (X509*)cert->direct.cert;
+            rc = SSL_CTX_use_certificate(ctx, xcert);
+            if (rc != 1) {
+                status = GET_SSL_STATUS(ssock);
+                PJ_PERROR(1,(ssock->pool->obj_name, status,
+                             "Error setting direct certificate"));
+                SSL_CTX_free(ctx);
+                ossock->ossl_ctx = NULL;
+                return status;
+            } else {
+                cert_loaded = PJ_TRUE;
+                PJ_LOG(4,(ssock->pool->obj_name, "Direct certificate set"));
+            }
+        }
     }
 
+
     if (ssock->is_server) {
         char *p = NULL;
 
@@ -1454,7 +1509,7 @@ static pj_status_t init_ossl_ctx(pj_ssl_sock_t *ssock)
             int i;
 
             /* Check and load ECC & DSA certificates & private keys */
-            for (i = 0; i < PJ_ARRAY_SIZE(cert_types); ++i) {
+            for (i = 0; i < (int)PJ_ARRAY_SIZE(cert_types); ++i) {
                 int err;
 
                 pj_memcpy(p, cert_types[i], CERT_TYPE_LEN);
@@ -1532,7 +1587,7 @@ static pj_status_t init_ossl_ctx(pj_ssl_sock_t *ssock)
             X509_NAME *xn = NULL;
             STACK_OF(X509_NAME) *sk = NULL;
             BIO *new_bio = BIO_new_mem_buf((void*)cert->CA_buf.ptr,
-                                           cert->CA_buf.slen);
+                                           (int)cert->CA_buf.slen);
 
             sk = sk_X509_NAME_new(xname_cmp);
 
@@ -1571,7 +1626,7 @@ static pj_status_t init_ossl_ctx(pj_ssl_sock_t *ssock)
             PJ_LOG(4,(ssock->pool->obj_name,
                       "CA certificates loaded from %s",
                       (cert->CA_file.slen?cert->CA_file.ptr:"buffer")));
-        } else {
+        } else if (cert->CA_file.slen > 0 || cert->CA_buf.slen > 0) {
             PJ_LOG(1,(ssock->pool->obj_name,
                       "Error reading CA certificates from %s",
                       (cert->CA_file.slen?cert->CA_file.ptr:"buffer")));
@@ -1604,8 +1659,10 @@ static pj_status_t ssl_create(pj_ssl_sock_t *ssock)
 
     set_entropy(ssock);
 
-    if (ssock->param.proto == PJ_SSL_SOCK_PROTO_DEFAULT)
-        ssock->param.proto = PJ_SSL_SOCK_PROTO_SSL23;
+    if (ssock->param.proto == PJ_SSL_SOCK_PROTO_DEFAULT) {
+        ssock->param.proto = PJ_SSL_SOCK_PROTO_TLS1_2 |
+                             PJ_SSL_SOCK_PROTO_TLS1_3;
+    }
 
     /* Create SSL context */
     if (SERVER_SUPPORT_SESSION_REUSE && ssock->is_server) {
@@ -1668,6 +1725,23 @@ static void ssl_destroy(pj_ssl_sock_t *ssock)
 {
     ossl_sock_t *ossock = (ossl_sock_t *)ssock;
 
+    /* For OpenSSL version >= 3.0, dec ref EVP_PKEY & X509 */
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+    if (ssock->cert && ssock->cert->direct.privkey &&
+        (ssock->cert->direct.type & PJ_SSL_CERT_DIRECT_OPENSSL_EVP_PKEY))
+    {
+        EVP_PKEY_free(ssock->cert->direct.privkey);
+        ssock->cert->direct.privkey = NULL;
+    }
+
+    if (ssock->cert && ssock->cert->direct.cert &&
+        (ssock->cert->direct.type & PJ_SSL_CERT_DIRECT_OPENSSL_X509_CERT))
+    {
+        X509_free(ssock->cert->direct.cert);
+        ssock->cert->direct.cert = NULL;
+    }
+#endif
+
     /* Destroy SSL instance */
     if (ossock->ossl_ssl) {
         SSL_free(ossock->ossl_ssl); /* this will also close BIOs */
@@ -1695,8 +1769,16 @@ static void ssl_destroy(pj_ssl_sock_t *ssock)
 /* Reset SSL socket state */
 static void ssl_reset_sock_state(pj_ssl_sock_t *ssock)
 {
+    int post_unlock_flush_circ_buf = 0;
+
     ossl_sock_t *ossock = (ossl_sock_t *)ssock;
 
+    /* Must lock around SSL calls, particularly SSL_shutdown
+     * as it can modify the write BIOs and destructively
+     * interfere with any ssl_write() calls in progress
+     * above in a multithreaded environment */
+    pj_lock_acquire(ssock->write_mutex);
+
     /* Detach from SSL instance */
     if (ossock->ossl_ssl) {
         SSL_set_ex_data(ossock->ossl_ssl, sslsock_idx, NULL);
@@ -1706,19 +1788,32 @@ static void ssl_reset_sock_state(pj_ssl_sock_t *ssock)
      * Avoid calling SSL_shutdown() if handshake wasn't completed.
      * OpenSSL 1.0.2f complains if SSL_shutdown() is called during an
      * SSL handshake, while previous versions always return 0.
+     * Call SSL_shutdown() when there is a timeout handshake failure or
+     * the last error is not SSL_ERROR_SYSCALL and not SSL_ERROR_SSL.
      */
     if (ossock->ossl_ssl && SSL_in_init(ossock->ossl_ssl) == 0) {
-        int ret = SSL_shutdown(ossock->ossl_ssl);
-        if (ret == 0) {
-            /* Flush data to send close notify. */
-            flush_circ_buf_output(ssock, &ssock->shutdown_op_key, 0, 0);
+        if (ssock->handshake_status == PJ_ETIMEDOUT ||
+            (ssock->last_err != SSL_ERROR_SYSCALL &&
+             ssock->last_err != SSL_ERROR_SSL))
+        {
+            int ret = SSL_shutdown(ossock->ossl_ssl);
+            if (ret == 0) {
+                /* SSL_shutdown will potentially trigger a bunch of
+                 * data to dump to the socket */
+                post_unlock_flush_circ_buf = 1;
+            }
         }
     }
 
-    pj_lock_acquire(ssock->write_mutex);
     ssock->ssl_state = SSL_STATE_NULL;
+
     pj_lock_release(ssock->write_mutex);
 
+    if (post_unlock_flush_circ_buf) {
+        /* Flush data to send close notify. */
+        flush_circ_buf_output(ssock, &ssock->shutdown_op_key, 0, 0);
+    }
+
     ssl_close_sockets(ssock);
 
     /* Upon error, OpenSSL may leave any error description in the thread 
@@ -1762,7 +1857,7 @@ static pj_status_t set_cipher_list(pj_ssl_sock_t *ssock)
     enum { BUF_SIZE = 8192 };
     pj_str_t cipher_list;
     unsigned i, j;
-    int ret;
+    int ret, ret2 = 1;
 
     if (ssock->param.ciphers_num == 0) {
         ret = SSL_CTX_set_cipher_list(ossock->ossl_ctx, PJ_SSL_SOCK_OSSL_CIPHERS);
@@ -1812,9 +1907,17 @@ static pj_status_t set_cipher_list(pj_ssl_sock_t *ssock)
     /* Put NULL termination in the generated cipher list */
     cipher_list.ptr[cipher_list.slen] = '\0';
 
-    /* Finally, set chosen cipher list */
+    /* Finally, set chosen cipher list.
+     * SSL_CTX_set_cipher_list() is for TLSv1.2 and below, while
+     * SSL_CTX_set_ciphersuites() is for TLSv1.3.
+     */
     ret = SSL_CTX_set_cipher_list(ossock->ossl_ctx, buf);
-    if (ret < 1) {
+#if !USING_BORINGSSL
+    ret2 = SSL_CTX_set_ciphersuites(ossock->ossl_ctx, buf);
+#endif
+    if (ret < 1 && ret2 < 1) {
+        PJ_LOG(4, (THIS_FILE, "Failed setting cipher list %s",
+                              cipher_list.ptr));
         pj_pool_release(tmp_pool);
         return GET_SSL_STATUS(ssock);
     }
@@ -2329,6 +2432,10 @@ static pj_status_t ssl_do_handshake(pj_ssl_sock_t *ssock)
 
     /* Perform SSL handshake */
     pj_lock_acquire(ssock->write_mutex);
+
+    /* Clear the error queue prior to any I/O functions, as per openssl docs */
+    ERR_clear_error();
+
     err = SSL_do_handshake(ossock->ossl_ssl);
     pj_lock_release(ssock->write_mutex);
 
@@ -2365,14 +2472,14 @@ static pj_status_t ssl_do_handshake(pj_ssl_sock_t *ssock)
 
 #if OPENSSL_VERSION_NUMBER >= 0x1010100fL
             PJ_LOG(5, (THIS_FILE, "Session info: reused=%d, resumable=%d, "
-                       "timeout=%d",
-                       SSL_session_reused(ossock->ossl_ssl),
+                       "timeout=%ld",
+                       (int)SSL_session_reused(ossock->ossl_ssl),
                        SSL_SESSION_is_resumable(sess),
                        SSL_SESSION_get_timeout(sess)));
 #else
             PJ_LOG(5, (THIS_FILE, "Session info: reused=%d, resumable=%d, "
-                       "timeout=%d",
-                       SSL_session_reused(ossock->ossl_ssl),
+                       "timeout=%ld",
+                       (int)SSL_session_reused(ossock->ossl_ssl),
                        -1,
                        SSL_SESSION_get_timeout(sess)));
 #endif
@@ -2381,14 +2488,14 @@ static pj_status_t ssl_do_handshake(pj_ssl_sock_t *ssock)
             len *= 2;
             if (len >= BUF_SIZE) len = BUF_SIZE;
             for (i = 0; i < len; i+=2)
-                pj_ansi_sprintf(buf+i, "%02X", sid[i/2]);
+                pj_ansi_snprintf(buf+i, sizeof(buf)-i, "%02X", sid[i/2]);
             buf[len] = '\0';
             PJ_LOG(5, (THIS_FILE, "Session id: %s", buf));
 
             sctx = SSL_SESSION_get0_id_context(sess, &len);
             if (len >= BUF_SIZE) len = BUF_SIZE;
             for (i = 0; i < len; i++)
-                pj_ansi_sprintf(buf + i, "%d", sctx[i]);
+                pj_ansi_snprintf(buf + i, sizeof(buf)-i, "%d", sctx[i]);
             buf[len] = '\0';
             PJ_LOG(5, (THIS_FILE, "Session id context: %s", buf));
         }
@@ -2412,8 +2519,11 @@ static pj_status_t ssl_read(pj_ssl_sock_t *ssock, void *data, int *size)
      * is on progress, so let's protect it with write mutex.
      */
     pj_lock_acquire(ssock->write_mutex);
+
+    /* Clear the error queue prior to any I/O functions, as per openssl docs */
+    ERR_clear_error();
+
     *size = size_ = SSL_read(ossock->ossl_ssl, data, size_);
-    pj_lock_release(ssock->write_mutex);
 
     if (size_ <= 0) {
         pj_status_t status;
@@ -2422,7 +2532,9 @@ static pj_status_t ssl_read(pj_ssl_sock_t *ssock, void *data, int *size)
         /* SSL might just return SSL_ERROR_WANT_READ in 
          * re-negotiation.
          */
-        if (err != SSL_ERROR_NONE && err != SSL_ERROR_WANT_READ) {
+        if (err != SSL_ERROR_NONE && err != SSL_ERROR_WANT_READ &&
+            err != SSL_ERROR_ZERO_RETURN)
+        {
             if (err == SSL_ERROR_SYSCALL && size_ == -1 &&
                 ERR_peek_error() == 0 && errno == 0)
             {
@@ -2436,15 +2548,24 @@ static pj_status_t ssl_read(pj_ssl_sock_t *ssock, void *data, int *size)
                 /* Reset SSL socket state, then return PJ_FALSE */
                 status = STATUS_FROM_SSL_ERR2("Read", ssock, size_,
                                               err, len);
+                pj_lock_release(ssock->write_mutex);
+                /* Unfortunately we can't hold the lock here to reset all the state.
+                  * We probably should though.
+                  */
                 ssl_reset_sock_state(ssock);
                 return status;
             }
         }
         
-        /* Need renegotiation */
-        return PJ_EEOF;
+        /* Return PJ_ETRYAGAIN when SSL needs renegotiation */
+        if (!SSL_is_init_finished(ossock->ossl_ssl)) {
+            pj_lock_release(ssock->write_mutex);
+            return PJ_ETRYAGAIN;
+        }
     }
 
+    pj_lock_release(ssock->write_mutex);
+
     return PJ_SUCCESS;
 }
 
@@ -2456,6 +2577,9 @@ static pj_status_t ssl_write(pj_ssl_sock_t *ssock, const void *data,
     ossl_sock_t *ossock = (ossl_sock_t *)ssock;
     pj_status_t status = PJ_SUCCESS;
 
+    /* Clear the error queue prior to any I/O functions, as per openssl docs */
+    ERR_clear_error();
+
     *nwritten = SSL_write(ossock->ossl_ssl, data, (int)size);
     if (*nwritten <= 0) {
         /* SSL failed to process the data, it may just that re-negotiation
@@ -2464,11 +2588,11 @@ static pj_status_t ssl_write(pj_ssl_sock_t *ssock, const void *data,
         int err;
         err = SSL_get_error(ossock->ossl_ssl, *nwritten);
         if (err == SSL_ERROR_WANT_READ || err == SSL_ERROR_NONE) {
-            status = PJ_EEOF;
+            status = PJ_ETRYAGAIN;
         } else {
-            /* Some problem occured */
+            /* Some problem occurred */
             status = STATUS_FROM_SSL_ERR2("Write", ssock, *nwritten,
-                                          err, size);
+                                          err, (int)size);
         }
     } else if (*nwritten < size) {
         /* nwritten < size, shouldn't happen, unless write BIO cannot hold 
@@ -2490,6 +2614,9 @@ static pj_status_t ssl_renegotiate(pj_ssl_sock_t *ssock)
     if (SSL_renegotiate_pending(ossock->ossl_ssl))
         return PJ_EPENDING;
 
+    /* Clear the error queue prior to any I/O functions, as per openssl docs */
+    ERR_clear_error();
+
     ret = SSL_renegotiate(ossock->ossl_ssl);
     if (ret <= 0) {
         status = GET_SSL_STATUS(ssock);
