@@ -1,4 +1,3 @@
-/* $Id$ */
 /*
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  *
@@ -39,29 +38,44 @@
 
 
 #if defined(PJMEDIA_HAS_VIDEO) && PJMEDIA_HAS_VIDEO != 0 && \
+    defined(PJMEDIA_HAS_LIBAVDEVICE) && PJMEDIA_HAS_LIBAVDEVICE != 0 && \
     defined(PJMEDIA_VIDEO_DEV_HAS_FFMPEG) && PJMEDIA_VIDEO_DEV_HAS_FFMPEG != 0
 
 
-#define THIS_FILE		"ffmpeg.c"
+#define THIS_FILE               "ffmpeg.c"
+
+#define LIBAVFORMAT_VER_AT_LEAST(major,minor)  (LIBAVFORMAT_VERSION_MAJOR > major || \
+                                               (LIBAVFORMAT_VERSION_MAJOR == major && \
+                                                LIBAVFORMAT_VERSION_MINOR >= minor))
 
 #include "../pjmedia/ffmpeg_util.h"
 #include <libavdevice/avdevice.h>
 #include <libavformat/avformat.h>
+#if LIBAVFORMAT_VER_AT_LEAST(53,2)
+# include <libavutil/pixdesc.h>
+#endif
+
+#define MAX_DEV_CNT      8
+#define MAX_DEV_NAME_LEN 80
+
+#ifndef PJMEDIA_USE_OLD_FFMPEG
+#  define av_free_packet av_packet_unref
+#  define av_close_input_stream(ctx) avformat_close_input(&ctx)
+#endif
 
-#define MAX_DEV_CNT     8
 
 typedef struct ffmpeg_dev_info
 {
     pjmedia_vid_dev_info         base;
-    AVInputFormat               *host_api;
+    const AVInputFormat         *host_api;
     const char                  *def_devname;
 } ffmpeg_dev_info;
 
 
 typedef struct ffmpeg_factory
 {
-    pjmedia_vid_dev_factory	 base;
-    pj_pool_factory		*pf;
+    pjmedia_vid_dev_factory      base;
+    pj_pool_factory             *pf;
     pj_pool_t                   *pool;
     pj_pool_t                   *dev_pool;
     unsigned                     dev_count;
@@ -76,6 +90,7 @@ typedef struct ffmpeg_stream
     pj_pool_t                   *pool;
     pjmedia_vid_dev_param        param;
     AVFormatContext             *ff_fmt_ctx;
+    void                        *frame_buf;
 } ffmpeg_stream;
 
 
@@ -85,27 +100,27 @@ static pj_status_t ffmpeg_factory_destroy(pjmedia_vid_dev_factory *f);
 static pj_status_t ffmpeg_factory_refresh(pjmedia_vid_dev_factory *f);
 static unsigned    ffmpeg_factory_get_dev_count(pjmedia_vid_dev_factory *f);
 static pj_status_t ffmpeg_factory_get_dev_info(pjmedia_vid_dev_factory *f,
-					       unsigned index,
-					       pjmedia_vid_dev_info *info);
+                                               unsigned index,
+                                               pjmedia_vid_dev_info *info);
 static pj_status_t ffmpeg_factory_default_param(pj_pool_t *pool,
                                                 pjmedia_vid_dev_factory *f,
-					        unsigned index,
-					        pjmedia_vid_dev_param *param);
+                                                unsigned index,
+                                                pjmedia_vid_dev_param *param);
 static pj_status_t ffmpeg_factory_create_stream(
-					pjmedia_vid_dev_factory *f,
-					pjmedia_vid_dev_param *param,
-					const pjmedia_vid_dev_cb *cb,
-					void *user_data,
-					pjmedia_vid_dev_stream **p_vid_strm);
+                                        pjmedia_vid_dev_factory *f,
+                                        pjmedia_vid_dev_param *param,
+                                        const pjmedia_vid_dev_cb *cb,
+                                        void *user_data,
+                                        pjmedia_vid_dev_stream **p_vid_strm);
 
 static pj_status_t ffmpeg_stream_get_param(pjmedia_vid_dev_stream *strm,
-					   pjmedia_vid_dev_param *param);
+                                           pjmedia_vid_dev_param *param);
 static pj_status_t ffmpeg_stream_get_cap(pjmedia_vid_dev_stream *strm,
-				         pjmedia_vid_dev_cap cap,
-				         void *value);
+                                         pjmedia_vid_dev_cap cap,
+                                         void *value);
 static pj_status_t ffmpeg_stream_set_cap(pjmedia_vid_dev_stream *strm,
-				         pjmedia_vid_dev_cap cap,
-				         const void *value);
+                                         pjmedia_vid_dev_cap cap,
+                                         const void *value);
 static pj_status_t ffmpeg_stream_start(pjmedia_vid_dev_stream *strm);
 static pj_status_t ffmpeg_stream_get_frame(pjmedia_vid_dev_stream *s,
                                            pjmedia_frame *frame);
@@ -148,40 +163,72 @@ static void print_ffmpeg_err(int err)
 static void print_ffmpeg_log(void* ptr, int level, const char* fmt, va_list vl)
 {
     PJ_UNUSED_ARG(ptr);
-    PJ_UNUSED_ARG(level);
+
+    /* Custom callback needs to filter log level by itself */
+    if (level > av_log_get_level())
+        return;
+
     vfprintf(stdout, fmt, vl);
 }
 
 
 static pj_status_t ffmpeg_capture_open(AVFormatContext **ctx,
-                                       AVInputFormat *ifmt,
+                                       const AVInputFormat *ifmt,
                                        const char *dev_name,
                                        const pjmedia_vid_dev_param *param)
 {
+#if LIBAVFORMAT_VER_AT_LEAST(53,2)
+    AVDictionary *format_opts = NULL;
+    char buf[128];
+    enum AVPixelFormat av_fmt;
+    
+    PJ_UNUSED_ARG(buf);
+#else
     AVFormatParameters fp;
+#endif
     pjmedia_video_format_detail *vfd;
+    pj_status_t status;
     int err;
 
     PJ_ASSERT_RETURN(ctx && ifmt && dev_name && param, PJ_EINVAL);
     PJ_ASSERT_RETURN(param->fmt.detail_type == PJMEDIA_FORMAT_DETAIL_VIDEO,
                      PJ_EINVAL);
 
+    status = pjmedia_format_id_to_PixelFormat(param->fmt.id, &av_fmt);
+    if (status != PJ_SUCCESS) {
+        avformat_free_context(*ctx);
+        return status;
+    }
+
     vfd = pjmedia_format_get_video_format_detail(&param->fmt, PJ_TRUE);
 
     /* Init ffmpeg format context */
     *ctx = avformat_alloc_context();
 
+#if LIBAVFORMAT_VER_AT_LEAST(53,2)
+    /* Init ffmpeg dictionary */
+    /*
+    snprintf(buf, sizeof(buf), "%d/%d", vfd->fps.num, vfd->fps.denum);
+    av_dict_set(&format_opts, "framerate", buf, 0);
+    snprintf(buf, sizeof(buf), "%dx%d", vfd->size.w, vfd->size.h);
+    av_dict_set(&format_opts, "video_size", buf, 0);
+    av_dict_set(&format_opts, "pixel_format", av_get_pix_fmt_name(av_fmt), 0);
+    */
+    /* Open capture stream */
+    err = avformat_open_input(ctx, dev_name, ifmt, &format_opts);
+#else
     /* Init ffmpeg format param */
     pj_bzero(&fp, sizeof(fp));
     fp.prealloced_context = 1;
     fp.width = vfd->size.w;
     fp.height = vfd->size.h;
-    fp.pix_fmt = PIX_FMT_BGR24;
+    fp.pix_fmt = av_fmt;
     fp.time_base.num = vfd->fps.denum;
     fp.time_base.den = vfd->fps.num;
 
     /* Open capture stream */
     err = av_open_input_stream(ctx, NULL, dev_name, ifmt, &fp);
+#endif
     if (err < 0) {
         *ctx = NULL; /* ffmpeg freed its states on failure, do we must too */
         print_ffmpeg_err(err);
@@ -244,60 +291,221 @@ static pj_status_t ffmpeg_factory_destroy(pjmedia_vid_dev_factory *f)
     return PJ_SUCCESS;
 }
 
+
+#if (defined(PJ_WIN32) && PJ_WIN32!=0) || \
+    (defined(PJ_WIN64) && PJ_WIN64!=0)
+
+#ifdef _MSC_VER
+#   pragma warning(push, 3)
+#endif
+
+#define COBJMACROS
+#include <DShow.h>
+#pragma comment(lib, "Strmiids.lib")
+
+#ifdef _MSC_VER
+#   pragma warning(pop)
+#endif
+
+static pj_status_t dshow_enum_devices(unsigned *dev_cnt,
+                                      char dev_names[][MAX_DEV_NAME_LEN])
+{
+    unsigned max_cnt = *dev_cnt;
+    ICreateDevEnum *dev_enum = NULL;
+    IEnumMoniker *enum_cat = NULL;
+    IMoniker *moniker = NULL;
+    HRESULT hr;
+    ULONG fetched;
+    unsigned i = 0;
+
+    CoInitialize(0);
+
+    *dev_cnt = 0;
+    hr = CoCreateInstance(&CLSID_SystemDeviceEnum, NULL,
+                          CLSCTX_INPROC_SERVER, &IID_ICreateDevEnum,
+                          (void**)&dev_enum);
+    if (FAILED(hr) ||
+        ICreateDevEnum_CreateClassEnumerator(dev_enum,
+            &CLSID_VideoInputDeviceCategory, &enum_cat, 0) != S_OK) 
+    {
+        PJ_LOG(4,(THIS_FILE, "Windows found no video input devices"));
+        if (dev_enum)
+            ICreateDevEnum_Release(dev_enum);
+
+        return PJ_SUCCESS;
+    }
+
+    while (IEnumMoniker_Next(enum_cat, 1, &moniker, &fetched) == S_OK &&
+           *dev_cnt < max_cnt)
+    {
+        (*dev_cnt)++;
+    }
+
+    if (*dev_cnt == 0) {
+        IEnumMoniker_Release(enum_cat);
+        ICreateDevEnum_Release(dev_enum);
+        return PJ_SUCCESS;
+    }
+
+    IEnumMoniker_Reset(enum_cat);
+    while (i < max_cnt &&
+           IEnumMoniker_Next(enum_cat, 1, &moniker, &fetched) == S_OK)
+    {
+        IPropertyBag *prop_bag;
+
+        hr = IMoniker_BindToStorage(moniker, 0, 0, &IID_IPropertyBag,
+                                    (void**)&prop_bag);
+        if (SUCCEEDED(hr)) {
+            VARIANT var_name;
+
+            VariantInit(&var_name);
+            hr = IPropertyBag_Read(prop_bag, L"FriendlyName", &var_name,
+                                   NULL);
+            if (SUCCEEDED(hr) && var_name.bstrVal) {
+                char tmp[MAX_DEV_NAME_LEN] = {0};
+                WideCharToMultiByte(CP_ACP, 0, var_name.bstrVal,
+                                    (int)wcslen(var_name.bstrVal),
+                                    tmp, MAX_DEV_NAME_LEN, NULL, NULL);
+                pj_ansi_snprintf(dev_names[i++], MAX_DEV_NAME_LEN,
+                                 "video=%s", tmp);
+            }
+            VariantClear(&var_name);
+            IPropertyBag_Release(prop_bag);
+        }
+        IMoniker_Release(moniker);
+    }
+
+    IEnumMoniker_Release(enum_cat);
+    ICreateDevEnum_Release(dev_enum);
+
+    PJ_LOG(4, (THIS_FILE, "DShow has %d devices:", *dev_cnt));
+    for (i = 0; i < *dev_cnt; ++i) {
+        PJ_LOG(4, (THIS_FILE, " %d: %s", (i+1), dev_names[i]));
+    }
+
+    return PJ_SUCCESS;
+}
+
+#endif /* PJ_WIN32 or PJ_WIN64 */
+
+
 /* API: refresh the list of devices */
 static pj_status_t ffmpeg_factory_refresh(pjmedia_vid_dev_factory *f)
 {
     ffmpeg_factory *ff = (ffmpeg_factory*)f;
-    AVInputFormat *p;
-    ffmpeg_dev_info *info;
+    const AVInputFormat *p;
 
     av_log_set_callback(&print_ffmpeg_log);
-    av_log_set_level(AV_LOG_DEBUG);
+    av_log_set_level(AV_LOG_ERROR);
 
     if (ff->dev_pool) {
         pj_pool_release(ff->dev_pool);
         ff->dev_pool = NULL;
     }
 
-    /* TODO: this should enumerate devices, now it enumerates host APIs */
     ff->dev_count = 0;
     ff->dev_pool = pj_pool_create(ff->pf, "ffmpeg_cap_dev", 500, 500, NULL);
 
-    p = av_iformat_next(NULL);
-    while (p) {
-        if (p->flags & AVFMT_NOFILE) {
+    /* Iterate host APIs */
+    p = av_input_video_device_next(NULL);
+    while (p && ff->dev_count < MAX_DEV_CNT) {
+        char dev_names[MAX_DEV_CNT][MAX_DEV_NAME_LEN];
+        unsigned dev_cnt = MAX_DEV_CNT;
+        unsigned dev_idx;
+
+#if LIBAVFORMAT_VER_AT_LEAST(60, 21)
+        if ((p->flags & AVFMT_NOFILE)==0) {
+#else
+        if ((p->flags & AVFMT_NOFILE)==0 || p->read_probe) {
+#endif
+            goto next_format;
+        }
+
+#if (defined(PJ_WIN32) && PJ_WIN32!=0) || \
+    (defined(PJ_WIN64) && PJ_WIN64!=0)
+        if (pj_ansi_strcmp(p->name, "dshow") == 0) {
+            dshow_enum_devices(&dev_cnt, dev_names);
+        } else if (pj_ansi_strcmp(p->name, "vfwcap") == 0) {
+            dev_cnt = 1;
+            pj_ansi_snprintf(dev_names[0], MAX_DEV_NAME_LEN, "0");
+        } else {
+            dev_cnt = 0;
+        }
+#elif defined(PJ_LINUX) && PJ_LINUX!=0
+        dev_cnt = 1;
+        pj_ansi_snprintf(dev_names[0], MAX_DEV_NAME_LEN, "/dev/video0");
+#else
+        dev_cnt = 0;
+#endif
+
+        /* Iterate devices (only DirectShow devices for now) */
+        for (dev_idx = 0; dev_idx < dev_cnt && ff->dev_count < MAX_DEV_CNT;
+            ++dev_idx)
+        {
+            ffmpeg_dev_info *info;
+            AVFormatContext *ctx;
+            AVCodecContext *codec = NULL;
+            pjmedia_format_id fmt_id;
+            pj_str_t dev_name;
+            pj_status_t status;
             unsigned i;
+            
+            ctx = avformat_alloc_context();
+            if (!ctx || avformat_open_input(&ctx, dev_names[dev_idx], p, NULL)!=0)
+                continue;
+
+            for(i = 0; i < ctx->nb_streams; i++) {
+#if LIBAVFORMAT_VER_AT_LEAST(59,0)
+                if (ctx->streams[i]->codecpar->codec_type == AVMEDIA_TYPE_VIDEO) {
+                    codec = ctx->streams[i]->codecpar;
+                    break;
+                }
+#else
+                if (ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO) {
+                    codec = ctx->streams[i]->codec;
+                    break;
+                }
+#endif
+            }
+            if (!codec) {
+                av_close_input_stream(ctx);
+                continue;
+            }
+
+            status = PixelFormat_to_pjmedia_format_id(codec->pix_fmt, &fmt_id);
+            if (status != PJ_SUCCESS) {
+                av_close_input_stream(ctx);
+                continue;
+            }
 
             info = &ff->dev_info[ff->dev_count++];
             pj_bzero(info, sizeof(*info));
-            pj_ansi_strncpy(info->base.name, "default", 
+            pj_ansi_strxcpy(info->base.name, "default", 
                             sizeof(info->base.name));
             pj_ansi_snprintf(info->base.driver, sizeof(info->base.driver),
-                             "%s (ffmpeg)", p->name);
+                             "ffmpeg %s", p->name);
+            
+            pj_strdup2_with_null(ff->pool, &dev_name, dev_names[dev_idx]);
+            info->def_devname = dev_name.ptr;
             info->base.dir = PJMEDIA_DIR_CAPTURE;
             info->base.has_callback = PJ_FALSE;
 
             info->host_api = p;
 
-#if (defined(PJ_WIN32) && PJ_WIN32!=0) || \
-    (defined(PJ_WIN64) && PJ_WIN64!=0)
-            info->def_devname = "0";
-#elif defined(PJ_LINUX) && PJ_LINUX!=0
-            info->def_devname = "/dev/video0";
-#endif
-
-            /* Set supported formats, currently hardcoded to RGB24 only */
+            /* Set supported formats */
             info->base.caps = PJMEDIA_VID_DEV_CAP_FORMAT;
             info->base.fmt_cnt = 1;
             for (i = 0; i < info->base.fmt_cnt; ++i) {
                 pjmedia_format *fmt = &info->base.fmt[i];
-
-                fmt->id = PJMEDIA_FORMAT_RGB24;
-                fmt->type = PJMEDIA_TYPE_VIDEO;
-                fmt->detail_type = PJMEDIA_FORMAT_DETAIL_NONE;
+                pjmedia_format_init_video(fmt, fmt_id,
+                                          codec->width, codec->height, 15, 1);
             }
+
+            av_close_input_stream(ctx);
         }
-        p = av_iformat_next(p);
+
+next_format:
+        p = av_input_video_device_next(p);
     }
 
     return PJ_SUCCESS;
@@ -312,8 +520,8 @@ static unsigned ffmpeg_factory_get_dev_count(pjmedia_vid_dev_factory *f)
 
 /* API: get device info */
 static pj_status_t ffmpeg_factory_get_dev_info(pjmedia_vid_dev_factory *f,
-					       unsigned index,
-					       pjmedia_vid_dev_info *info)
+                                               unsigned index,
+                                               pjmedia_vid_dev_info *info)
 {
     ffmpeg_factory *ff = (ffmpeg_factory*)f;
 
@@ -327,8 +535,8 @@ static pj_status_t ffmpeg_factory_get_dev_info(pjmedia_vid_dev_factory *f,
 /* API: create default device parameter */
 static pj_status_t ffmpeg_factory_default_param(pj_pool_t *pool,
                                                 pjmedia_vid_dev_factory *f,
-					        unsigned index,
-					        pjmedia_vid_dev_param *param)
+                                                unsigned index,
+                                                pjmedia_vid_dev_param *param)
 {
     ffmpeg_factory *ff = (ffmpeg_factory*)f;
     ffmpeg_dev_info *info;
@@ -358,11 +566,11 @@ static pj_status_t ffmpeg_factory_default_param(pj_pool_t *pool,
 
 /* API: create stream */
 static pj_status_t ffmpeg_factory_create_stream(
-					pjmedia_vid_dev_factory *f,
-					pjmedia_vid_dev_param *param,
-					const pjmedia_vid_dev_cb *cb,
-					void *user_data,
-					pjmedia_vid_dev_stream **p_vid_strm)
+                                        pjmedia_vid_dev_factory *f,
+                                        pjmedia_vid_dev_param *param,
+                                        const pjmedia_vid_dev_cb *cb,
+                                        void *user_data,
+                                        pjmedia_vid_dev_stream **p_vid_strm)
 {
     ffmpeg_factory *ff = (ffmpeg_factory*)f;
     pj_pool_t *pool;
@@ -372,7 +580,7 @@ static pj_status_t ffmpeg_factory_create_stream(
     PJ_ASSERT_RETURN(param->dir == PJMEDIA_DIR_CAPTURE, PJ_EINVAL);
     PJ_ASSERT_RETURN((unsigned)param->cap_id < ff->dev_count, PJ_EINVAL);
     PJ_ASSERT_RETURN(param->fmt.detail_type == PJMEDIA_FORMAT_DETAIL_VIDEO,
-		     PJ_EINVAL);
+                     PJ_EINVAL);
 
     PJ_UNUSED_ARG(cb);
     PJ_UNUSED_ARG(user_data);
@@ -386,16 +594,35 @@ static pj_status_t ffmpeg_factory_create_stream(
     strm->pool = pool;
     pj_memcpy(&strm->param, param, sizeof(*param));
 
+    /* Allocate frame buffer */
+    {
+        const pjmedia_video_format_info *vfi;
+        pjmedia_video_apply_fmt_param vafp;
+
+        vfi = pjmedia_get_video_format_info(NULL, param->fmt.id);
+        if (!vfi) goto on_error;
+
+        pj_bzero(&vafp, sizeof(vafp));
+        vafp.size = param->fmt.det.vid.size;
+        vfi->apply_fmt(vfi, &vafp);
+
+        strm->frame_buf = pj_pool_alloc(pool, vafp.framebytes);
+    }
+
     /* Done */
     strm->base.op = &stream_op;
     *p_vid_strm = &strm->base;
 
     return PJ_SUCCESS;
+
+on_error:
+    pj_pool_release(pool);
+    return PJMEDIA_EVID_INVCAP;
 }
 
 /* API: Get stream info. */
 static pj_status_t ffmpeg_stream_get_param(pjmedia_vid_dev_stream *s,
-					   pjmedia_vid_dev_param *pi)
+                                           pjmedia_vid_dev_param *pi)
 {
     ffmpeg_stream *strm = (ffmpeg_stream*)s;
 
@@ -408,8 +635,8 @@ static pj_status_t ffmpeg_stream_get_param(pjmedia_vid_dev_stream *s,
 
 /* API: get capability */
 static pj_status_t ffmpeg_stream_get_cap(pjmedia_vid_dev_stream *s,
-				         pjmedia_vid_dev_cap cap,
-				         void *pval)
+                                         pjmedia_vid_dev_cap cap,
+                                         void *pval)
 {
     ffmpeg_stream *strm = (ffmpeg_stream*)s;
 
@@ -422,8 +649,8 @@ static pj_status_t ffmpeg_stream_get_cap(pjmedia_vid_dev_stream *s,
 
 /* API: set capability */
 static pj_status_t ffmpeg_stream_set_cap(pjmedia_vid_dev_stream *s,
-				         pjmedia_vid_dev_cap cap,
-				         const void *pval)
+                                         pjmedia_vid_dev_cap cap,
+                                         const void *pval)
 {
     ffmpeg_stream *strm = (ffmpeg_stream*)s;
 
@@ -462,7 +689,7 @@ static pj_status_t ffmpeg_stream_get_frame(pjmedia_vid_dev_stream *s,
                                            pjmedia_frame *frame)
 {
     ffmpeg_stream *strm = (ffmpeg_stream*)s;
-    AVPacket p;
+    AVPacket p = {0};
     int err;
 
     err = av_read_frame(strm->ff_fmt_ctx, &p);
@@ -473,8 +700,10 @@ static pj_status_t ffmpeg_stream_get_frame(pjmedia_vid_dev_stream *s,
 
     pj_bzero(frame, sizeof(*frame));
     frame->type = PJMEDIA_FRAME_TYPE_VIDEO;
-    frame->buf = p.data;
+    frame->buf = strm->frame_buf;
     frame->size = p.size;
+    pj_memcpy(frame->buf, p.data, p.size);
+    av_free_packet(&p);
 
     return PJ_SUCCESS;
 }
@@ -515,4 +744,4 @@ static pj_status_t ffmpeg_stream_destroy(pjmedia_vid_dev_stream *s)
 #endif
 
 
-#endif	/* PJMEDIA_VIDEO_DEV_HAS_FFMPEG */
+#endif  /* PJMEDIA_VIDEO_DEV_HAS_FFMPEG */
