@@ -1,4 +1,3 @@
-# $Id$
 #
 # Object oriented PJSUA wrapper.
 #
@@ -56,8 +55,11 @@ tutorial. The paragraphs below explain basic tasks on using this module.
 
 
 """
+try:
+    import _thread as thread
+except ImportError:
+    import thread
 import _pjsua
-import thread
 import threading
 import weakref
 import time
@@ -487,6 +489,8 @@ class TransportConfig:
                           Voice traffic.
                      4: PJ_QOS_TYPE_CONTROL
                           Control traffic.
+                     5: PJ_QOS_TYPE_SIGNALLING
+                          Signalling traffic.
     qos_params_flags    -- Determines which values to set, bitmask of pj_qos_flag.
                              PJ_QOS_PARAM_HAS_DSCP = 1
                              PJ_QOS_PARAM_HAS_SO_PRIO = 2
@@ -603,17 +607,22 @@ class Transport:
             self._lib._err_check("info()", self, -1, "Invalid transport")
         return TransportInfo(ti)
 
+    # NOTE: enable() and disable() methods are commented out because the underlying
+    # pjsua_transport_set_enable function has been deprecated and is no longer available.
+    # Please use PJSUA2 SWIG binding for modern transport management functionality.
+    """
     def enable(self):
-        """Enable this transport."""
+        \"\"\"Enable this transport.\"\"\"
         lck = self._lib.auto_lock()
         err = _pjsua.transport_set_enable(self._id, True)
         self._lib._err_check("enable()", self, err)
 
     def disable(self):
-        """Disable this transport."""
+        \"\"\"Disable this transport.\"\"\"
         lck = self._lib.auto_lock()
         err = _pjsua.transport_set_enable(self._id, 0)
         self._lib._err_check("disable()", self, err)
+    """
 
     def close(self, force=False):
         """Close and destroy this transport.
@@ -983,9 +992,9 @@ class AccountCallback:
         """Notification about incoming call.
 
         Application should implement one of on_incoming_call() or
-	on_incoming_call2(), otherwise, the default behavior is to
+        on_incoming_call2(), otherwise, the default behavior is to
         reject the call with default status code. Note that if both are
-	implemented, only on_incoming_call2() will be called.
+        implemented, only on_incoming_call2() will be called.
 
         Keyword arguments:
         call    -- the new incoming call
@@ -996,16 +1005,16 @@ class AccountCallback:
         """Notification about incoming call, with received SIP message info.
 
         Application should implement one of on_incoming_call() or
-	on_incoming_call2(), otherwise, the default behavior is to
+        on_incoming_call2(), otherwise, the default behavior is to
         reject the call with default status code. Note that if both are
-	implemented, only on_incoming_call2() will be called.
+        implemented, only on_incoming_call2() will be called.
 
         Keyword arguments:
         call    -- the new incoming call
         rdata   -- the received message
         """
         call.hangup()
-	
+
     def on_incoming_subscribe(self, buddy, from_uri, contact_uri, pres_obj):
         """Notification when incoming SUBSCRIBE request is received. 
         
@@ -1080,8 +1089,8 @@ class AccountCallback:
     def on_mwi_info(self, body):
         """
         Notification about change in Message Summary / Message Waiting
-	Indication (RFC 3842) status. MWI subscription must be enabled
-	in the account config to receive this notification.
+        Indication (RFC 3842) status. MWI subscription must be enabled
+        in the account config to receive this notification.
 
         Keyword arguments:
         body      -- String containing message body as received in the
@@ -1792,7 +1801,7 @@ class Call:
         self._lib()._err_check("send_request()", self, err)
 
     def send_pager(self, text, im_id=0, content_type="text/plain", 
-    		   hdr_list=None):
+                   hdr_list=None):
         """Send instant message inside a call.
 
         Keyword arguments:
@@ -2133,14 +2142,14 @@ class _LibMutex:
     def __init__(self, lck):
         self._lck = lck
         self._lck.acquire()
-	#_Trace(('lock acquired',))
+        #_Trace(('lock acquired',))
 
     def __del__(self):
         try:
             self._lck.release()
-	    #_Trace(('lock released',))
+            #_Trace(('lock released',))
         except:
-	    #_Trace(('lock release error',))
+            #_Trace(('lock release error',))
             pass
 
 
@@ -2210,7 +2219,7 @@ class Lib:
         py_ua_cfg.cb.on_pager = _cb_on_pager
         py_ua_cfg.cb.on_pager_status = _cb_on_pager_status
         py_ua_cfg.cb.on_typing = _cb_on_typing
-	py_ua_cfg.cb.on_mwi_info = _cb_on_mwi_info;
+        py_ua_cfg.cb.on_mwi_info = _cb_on_mwi_info;
 
         err = _pjsua.init(py_ua_cfg, log_cfg._cvt_to_pjsua(), 
                           media_cfg._cvt_to_pjsua())
@@ -2258,18 +2267,18 @@ class Lib:
         return _pjsua.handle_events(timeout)
 
     def thread_register(self, name):
-	"""Register external threads (threads that are not created by PJSIP,
-	such as threads that are created by Python API) to PJSIP.
+        """Register external threads (threads that are not created by PJSIP,
+        such as threads that are created by Python API) to PJSIP.
 
-	The call must be made from the new thread before calling any pjlib 
-	functions.
+        The call must be made from the new thread before calling any pjlib
+        functions.
 
-	Keyword arguments:
-	name	-- Non descriptive name for the thread
-	"""
-	dummy = 1
-	err = _pjsua.thread_register(name, dummy)
-	self._err_check("thread_register()", self, err)
+        Keyword arguments:
+        name -- Non descriptive name for the thread
+        """
+        dummy = 1
+        err = _pjsua.thread_register(name, dummy)
+        self._err_check("thread_register()", self, err)
 
     def verify_sip_url(self, sip_url):
         """Verify that the specified string is a valid URI. 
@@ -2946,7 +2955,7 @@ def _worker_thread_main(arg):
     _lib._err_check("thread_register()", _lib, err)
     while _lib and _lib._quit == 0:
         _lib.handle_events(1)
-	time.sleep(0.050)
+        time.sleep(0.050)
     if _lib:
         _lib._quit = 2
     _Trace(('worker thread exited..',))
@@ -2954,8 +2963,8 @@ def _worker_thread_main(arg):
 def _Trace(args):
     global enable_trace
     if enable_trace:
-        print "** ",
+        print("** ", end='')
         for arg in args:
-            print arg,
-        print " **"
+            print(arg, end=' ')
+        print(" **")
 
