@@ -1,4 +1,3 @@
-# $Id: setup.py 4232 2012-08-20 06:01:41Z ming $
 #
 # pjsua Setup script.
 #
@@ -32,25 +31,25 @@ pj_version_suffix=""
 f = open('../../../version.mak', 'r')
 for line in f:
     if line.find("export PJ_VERSION_MAJOR") != -1:
-    	tokens=line.split("=")
-	if len(tokens)>1:
-		pj_version_major= tokens[1].strip()
+        tokens=line.split("=")
+        if len(tokens)>1:
+                pj_version_major= tokens[1].strip()
     elif line.find("export PJ_VERSION_MINOR") != -1:
-    	tokens=line.split("=")
-	if len(tokens)>1:
-		pj_version_minor= line.split("=")[1].strip()
+        tokens=line.split("=")
+        if len(tokens)>1:
+                pj_version_minor= line.split("=")[1].strip()
     elif line.find("export PJ_VERSION_REV") != -1:
-    	tokens=line.split("=")
-	if len(tokens)>1:
-		pj_version_rev= line.split("=")[1].strip()
+        tokens=line.split("=")
+        if len(tokens)>1:
+                pj_version_rev= line.split("=")[1].strip()
     elif line.find("export PJ_VERSION_SUFFIX") != -1:
-    	tokens=line.split("=")
-	if len(tokens)>1:
-		pj_version_suffix= line.split("=")[1].strip()
+        tokens=line.split("=")
+        if len(tokens)>1:
+                pj_version_suffix= line.split("=")[1].strip()
 
 f.close()
 if not pj_version_major:
-    print 'Unable to get PJ_VERSION_MAJOR'
+    print ( 'Unable to get PJ_VERSION_MAJOR' )
     sys.exit(1)
 
 pj_version = pj_version_major + "." + pj_version_minor
@@ -61,24 +60,26 @@ if pj_version_suffix:
 
 #print 'PJ_VERSION = "'+ pj_version + '"'
 
+# Get 'make' from environment variable if any
+MAKE = os.environ.get('MAKE') or "make"
 
 # Fill in pj_inc_dirs
 pj_inc_dirs = []
-f = os.popen("make -f helper.mak inc_dir")
+f = os.popen("%s -f helper.mak inc_dir" % MAKE)
 for line in f:
     pj_inc_dirs.append(line.rstrip("\r\n"))
 f.close()
 
 # Fill in pj_lib_dirs
 pj_lib_dirs = []
-f = os.popen("make -f helper.mak lib_dir")
+f = os.popen("%s -f helper.mak lib_dir" % MAKE)
 for line in f:
     pj_lib_dirs.append(line.rstrip("\r\n"))
 f.close()
 
 # Fill in pj_libs
 pj_libs = []
-f = os.popen("make -f helper.mak libs")
+f = os.popen("%s -f helper.mak libs" % MAKE)
 for line in f:
     pj_libs.append(line.rstrip("\r\n"))
 f.close()
