@@ -1,4 +1,3 @@
-/* $Id$ */
 /*
  * Copyright (C) 2012-2013 Teluu Inc. (http://www.teluu.com)
  *
@@ -16,7 +15,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-#include <pjsua2/account.hpp>
 #include <pjsua2/call.hpp>
 #include <pjsua2/endpoint.hpp>
 #include <pj/ctype.h>
@@ -27,10 +25,13 @@ using namespace std;
 
 #include <pjsua-lib/pjsua_internal.h>
 
-#define THIS_FILE		"call.cpp"
+#define THIS_FILE               "call.cpp"
 
 ///////////////////////////////////////////////////////////////////////////////
 
+/* Avoid conflict with predefined standard macros. */
+#undef max
+#undef min
 
 MathStat::MathStat()
 : n(0), max(0), min(0), last(0), mean(0)
@@ -130,13 +131,13 @@ void MediaTransportInfo::fromPj(const pjmedia_transport_info &info)
     localRtpName = localRtcpName = srcRtpName = srcRtcpName = "";    
     if (pj_sockaddr_has_addr(&info.sock_info.rtp_addr_name)) { 
         pj_sockaddr_print(&info.sock_info.rtp_addr_name, straddr, 
-		          sizeof(straddr), 3);
+                          sizeof(straddr), 3);
         localRtpName = straddr;
     }
 
     if (pj_sockaddr_has_addr(&info.sock_info.rtcp_addr_name)) { 
         pj_sockaddr_print(&info.sock_info.rtcp_addr_name, straddr, 
-		          sizeof(straddr), 3);
+                          sizeof(straddr), 3);
         localRtcpName = straddr;
     }
 
@@ -202,6 +203,11 @@ void CallSendDtmfParam::fromPj(const pjsua_call_send_dtmf_param &param)
     this->digits    = pj2Str(param.digits);
 }
 
+CallSendTextParam::CallSendTextParam()
+{
+    medIdx = -1;
+}
+
 CallSetting::CallSetting(bool useDefaultValues)
 {
     if (useDefaultValues) {
@@ -214,38 +220,70 @@ CallSetting::CallSetting(bool useDefaultValues)
         reqKeyframeMethod   = 0;
         audioCount          = 0;
         videoCount          = 0;
+        textCount           = 0;
     }
 }
 
 bool CallSetting::isEmpty() const
 {
     return (flag == 0 && reqKeyframeMethod == 0 && audioCount == 0 &&
-            videoCount == 0);
+            videoCount == 0 && textCount == 0);
 }
 
 void CallSetting::fromPj(const pjsua_call_setting &prm)
 {
+    int i, mi;
+
     this->flag              = prm.flag;
     this->reqKeyframeMethod = prm.req_keyframe_method;
     this->audioCount        = prm.aud_cnt;
     this->videoCount        = prm.vid_cnt;
+    this->textCount         = prm.txt_cnt;
+    this->mediaDir.clear();
+    this->customCallId      = pj2Str(prm.custom_call_id);
+
+    /* Since we don't know the size of media_dir array, we populate
+     * mediaDir vector up to the element with non-default value.
+     */
+    for (mi = PJMEDIA_MAX_SDP_MEDIA - 1; mi >= 0; mi--) {
+        if (prm.media_dir[mi] != PJMEDIA_DIR_ENCODING_DECODING) break;
+    }
+    for (i = 0; i <= mi; i++) {
+        this->mediaDir.push_back(prm.media_dir[i]);
+    }
 }
 
 pjsua_call_setting CallSetting::toPj() const
 {
     pjsua_call_setting setting;
+    unsigned mi;
+
+    /* This is important to initialize media_dir array. */
+    pjsua_call_setting_default(&setting);
 
     setting.flag                = this->flag;
     setting.req_keyframe_method = this->reqKeyframeMethod;
     setting.aud_cnt             = this->audioCount;
     setting.vid_cnt             = this->videoCount;
+    setting.txt_cnt             = this->textCount;
+    for (mi = 0; mi < this->mediaDir.size(); mi++) {
+        setting.media_dir[mi] = (pjmedia_dir)this->mediaDir[mi];
+    }
+
+    if( ! this->customCallId.empty()) {
+        setting.custom_call_id = str2Pj(this->customCallId);
+    }
     
     return setting;
 }
 
 
 CallMediaInfo::CallMediaInfo()
-: audioConfSlot(PJSUA_INVALID_ID),
+: index(0xFF),
+  type(PJMEDIA_TYPE_NONE),
+  dir(PJMEDIA_DIR_NONE),
+  status(PJSUA_CALL_MEDIA_NONE),
+  audioConfSlot(PJSUA_INVALID_ID),
   videoIncomingWindowId(PJSUA_INVALID_ID),
   videoWindow(PJSUA_INVALID_ID),
   videoCapDev(PJMEDIA_VID_INVALID_DEV)
@@ -271,7 +309,7 @@ void CallInfo::fromPj(const pjsua_call_info &pci)
 {
     unsigned mi;
     
-    id 			= pci.id;
+    id                  = pci.id;
     role                = pci.role;
     accId               = pci.acc_id;
     localUri            = pj2Str(pci.local_info);
@@ -289,6 +327,7 @@ void CallInfo::fromPj(const pjsua_call_info &pci)
     remOfferer          = PJ2BOOL(pci.rem_offerer);
     remAudioCount       = pci.rem_aud_cnt;
     remVideoCount       = pci.rem_vid_cnt;
+    remTextCount        = pci.rem_txt_cnt;
     
     for (mi = 0; mi < pci.media_cnt; mi++) {
         CallMediaInfo med;
@@ -318,9 +357,23 @@ void StreamInfo::fromPj(const pjsua_stream_info &info)
         remoteRtcpAddress = straddr;
         txPt = info.info.aud.tx_pt;
         rxPt = info.info.aud.rx_pt;
+        audTxEventPt = info.info.aud.tx_event_pt;
+        audRxEventPt = info.info.aud.rx_event_pt;
         codecName = pj2Str(info.info.aud.fmt.encoding_name);
         codecClockRate = info.info.aud.fmt.clock_rate;
         audCodecParam.fromPj(*info.info.aud.param);
+        jbInit = info.info.aud.jb_init;
+        jbMinPre = info.info.aud.jb_min_pre;
+        jbMaxPre = info.info.aud.jb_max_pre;
+        jbMax = info.info.aud.jb_max;
+        jbDiscardAlgo = info.info.aud.jb_discard_algo;
+#if defined(PJMEDIA_STREAM_ENABLE_KA) && (PJMEDIA_STREAM_ENABLE_KA != 0)
+        useKa = PJ2BOOL(info.info.aud.use_ka);
+        startCountKa = info.info.aud.ka_cfg.start_count;
+        startIntervalKa =  info.info.aud.ka_cfg.start_interval;
+        intervalKa = info.info.aud.ka_cfg.ka_interval;
+#endif
+        rtcpSdesByeDisabled = PJ2BOOL(info.info.aud.rtcp_sdes_bye_disabled);
     } else if (type == PJMEDIA_TYPE_VIDEO) {
         proto = info.info.vid.proto;
         dir = info.info.vid.dir;
@@ -331,8 +384,43 @@ void StreamInfo::fromPj(const pjsua_stream_info &info)
         txPt = info.info.vid.tx_pt;
         rxPt = info.info.vid.rx_pt;
         codecName = pj2Str(info.info.vid.codec_info.encoding_name);
-        codecClockRate = info.info.vid.codec_info.clock_rate;        
-	vidCodecParam.fromPj(*info.info.vid.codec_param);
+        codecClockRate = info.info.vid.codec_info.clock_rate;
+        vidCodecParam.fromPj(*info.info.vid.codec_param);
+        jbInit = info.info.vid.jb_init;
+        jbMinPre = info.info.vid.jb_min_pre;
+        jbMaxPre = info.info.vid.jb_max_pre;
+        jbMax = info.info.vid.jb_max;
+        jbDiscardAlgo = PJMEDIA_JB_DISCARD_NONE;
+#if defined(PJMEDIA_STREAM_ENABLE_KA) && (PJMEDIA_STREAM_ENABLE_KA != 0)
+        useKa = PJ2BOOL(info.info.vid.use_ka);
+        startCountKa = info.info.vid.ka_cfg.start_count;
+        startIntervalKa =  info.info.vid.ka_cfg.start_interval;
+        intervalKa = info.info.vid.ka_cfg.ka_interval;
+#endif
+        rtcpSdesByeDisabled = PJ2BOOL(info.info.vid.rtcp_sdes_bye_disabled);
+    } else if (type == PJMEDIA_TYPE_TEXT) {
+        proto = info.info.txt.proto;
+        dir = info.info.txt.dir;
+        pj_sockaddr_print(&info.info.txt.rem_addr, straddr, sizeof(straddr), 3);
+        remoteRtpAddress = straddr;
+        pj_sockaddr_print(&info.info.txt.rem_rtcp, straddr, sizeof(straddr), 3);
+        remoteRtcpAddress = straddr;
+        txPt = info.info.txt.tx_pt;
+        rxPt = info.info.txt.rx_pt;
+        codecName = pj2Str(info.info.txt.fmt.encoding_name);
+        codecClockRate = info.info.txt.fmt.clock_rate;
+        jbInit = info.info.txt.jb_init;
+        jbMinPre = info.info.txt.jb_min_pre;
+        jbMaxPre = info.info.txt.jb_max_pre;
+        jbMax = info.info.txt.jb_max;
+        jbDiscardAlgo = PJMEDIA_JB_DISCARD_NONE;
+#if defined(PJMEDIA_STREAM_ENABLE_KA) && (PJMEDIA_STREAM_ENABLE_KA != 0)
+        useKa = PJ2BOOL(info.info.txt.use_ka);
+        startCountKa = info.info.txt.ka_cfg.start_count;
+        startIntervalKa =  info.info.txt.ka_cfg.start_interval;
+        intervalKa = info.info.txt.ka_cfg.ka_interval;
+#endif
+        rtcpSdesByeDisabled = PJ2BOOL(info.info.txt.rtcp_sdes_bye_disabled);
     }
 }
 
@@ -342,15 +430,19 @@ void StreamStat::fromPj(const pjsua_stream_stat &prm)
     jbuf.fromPj(prm.jbuf);
 }
 
+void OnCallRxTextParam::fromPj(const pjsua_txt_stream_data &prm)
+{
+    seq = prm.seq;
+    ts = prm.ts;
+    text = pj2Str(prm.text);
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 
 struct call_param
 {
-    pjsua_msg_data      msg_data;
     pjsua_msg_data     *p_msg_data;
-    pjsua_call_setting  opt;
     pjsua_call_setting *p_opt;
-    pj_str_t            reason;
     pj_str_t           *p_reason;
     pjmedia_sdp_session *sdp;
 
@@ -362,10 +454,17 @@ public:
     call_param(const SipTxOption &tx_option, const CallSetting &setting,
                const string &reason_str, pj_pool_t *pool = NULL,
                const string &sdp_str = "");
+
+private:
+    /* internal storages */
+    pjsua_msg_data      msg_data;
+    pjsua_call_setting  opt;
+    pj_str_t            reason;
 };
 
 call_param::call_param(const SipTxOption &tx_option)
 {
+    pjsua_msg_data_init(&msg_data);
     if (tx_option.isEmpty()) {
         p_msg_data = NULL;
     } else {
@@ -403,16 +502,16 @@ call_param::call_param(const SipTxOption &tx_option, const CallSetting &setting,
     if (sdp_str != "") {
         pj_str_t dup_pj_sdp;
         pj_str_t pj_sdp_str = {(char*)sdp_str.c_str(),
-        		       (pj_ssize_t)sdp_str.size()};
-	pj_status_t status;
+                               (pj_ssize_t)sdp_str.size()};
+        pj_status_t status;
 
         pj_strdup(pool, &dup_pj_sdp, &pj_sdp_str);        
         status = pjmedia_sdp_parse(pool, dup_pj_sdp.ptr,
-				   dup_pj_sdp.slen, &sdp);
-	if (status != PJ_SUCCESS) {
-	    PJ_PERROR(4,(THIS_FILE, status,
-			 "Failed to parse SDP for call param"));
-	}
+                                   dup_pj_sdp.slen, &sdp);
+        if (status != PJ_SUCCESS) {
+            PJ_PERROR(4,(THIS_FILE, status,
+                         "Failed to parse SDP for call param"));
+        }
     }
 }
 
@@ -427,20 +526,20 @@ Call::~Call()
 {
     /* Remove reference to this instance from PJSUA library */
     if (id != PJSUA_INVALID_ID)
-	pjsua_call_set_user_data(id, NULL);
+        pjsua_call_set_user_data(id, NULL);
 
     /*
      * If this instance is deleted, also hangup the corresponding call in
      * PJSUA library.
      */
     if (pjsua_get_state() < PJSUA_STATE_CLOSING && isActive()) {
-	try {
-	    CallOpParam prm;
-	    hangup(prm);
-	} catch (Error &err) {
-	    // Ignore
-	    PJ_UNUSED_ARG(err);
-	}
+        try {
+            CallOpParam prm;
+            hangup(prm);
+        } catch (Error &err) {
+            // Ignore
+            PJ_UNUSED_ARG(err);
+        }
     }
 }
 
@@ -471,12 +570,16 @@ Call *Call::lookup(int call_id)
 {
     Call *call = (Call*)pjsua_call_get_user_data(call_id);
     if (call && call_id != call->id) {
-	if (call->child && call->child->id == PJSUA_INVALID_ID) {
-	    /* This must be a new call from call transfer */
-	    call = call->child;
-	    pjsua_call_set_user_data(call_id, call);
-	}
-	call->id = call_id;
+        if (call->child && call->child->id == PJSUA_INVALID_ID) {
+            /* This must be a new call from call transfer or call replace
+             * which initially shares user_data with its parent (so the
+             * user_data points to its parent's Call instance).
+             * Let's update its user_data to its own Call instance.
+             */
+            call = call->child;
+            pjsua_call_set_user_data(call_id, call);
+        }
+        call->id = call_id;
     }
     return call;
 }
@@ -505,31 +608,31 @@ AudioMedia Call::getAudioMedia(int med_idx) const PJSUA2_THROW(Error)
     pjsua_call_get_info(id, &pj_ci);
     
     if (med_idx < 0) {
-	for (unsigned i = 0; i < pj_ci.media_cnt; ++i) {
-	    if (pj_ci.media[i].type == PJMEDIA_TYPE_AUDIO &&
-		pj_ci.media[i].stream.aud.conf_slot != PJSUA_INVALID_ID)
-	    {
-		med_idx = i;
-		break;
-	    }
-	}
-	if (med_idx < 0) {
-    	    PJSUA2_RAISE_ERROR3(PJ_ENOTFOUND, "getAudioMedia()",
-				"no active audio media");
-	}
+        for (unsigned i = 0; i < pj_ci.media_cnt; ++i) {
+            if (pj_ci.media[i].type == PJMEDIA_TYPE_AUDIO &&
+                pj_ci.media[i].stream.aud.conf_slot != PJSUA_INVALID_ID)
+            {
+                med_idx = i;
+                break;
+            }
+        }
+        if (med_idx < 0) {
+            PJSUA2_RAISE_ERROR3(PJ_ENOTFOUND, "getAudioMedia()",
+                                "no active audio media");
+        }
     }
 
     if (med_idx >= (int)pj_ci.media_cnt) {
-    	PJSUA2_RAISE_ERROR3(PJ_EINVAL, "getAudioMedia()",
-			    "invalid media index");
+        PJSUA2_RAISE_ERROR3(PJ_EINVAL, "getAudioMedia()",
+                            "invalid media index");
     }
     if (pj_ci.media[med_idx].type != PJMEDIA_TYPE_AUDIO) {
-    	PJSUA2_RAISE_ERROR3(PJ_EINVAL, "getAudioMedia()",
-			    "media is not audio");
+        PJSUA2_RAISE_ERROR3(PJ_EINVAL, "getAudioMedia()",
+                            "media is not audio");
     }
     if (pj_ci.media[med_idx].stream.aud.conf_slot == PJSUA_INVALID_ID) {
-    	PJSUA2_RAISE_ERROR3(PJ_EINVAL, "getAudioMedia()",
-			    "no audio slot (inactive?)");
+        PJSUA2_RAISE_ERROR3(PJ_EINVAL, "getAudioMedia()",
+                            "no audio slot (inactive?)");
     }
 
     AudioMediaHelper am;
@@ -543,31 +646,31 @@ VideoMedia Call::getEncodingVideoMedia(int med_idx) const PJSUA2_THROW(Error)
     pjsua_call_get_info(id, &pj_ci);
 
     if (med_idx < 0) {
-	for (unsigned i = 0; i < pj_ci.media_cnt; ++i) {
-	    if (pj_ci.media[i].type == PJMEDIA_TYPE_VIDEO &&
-		pj_ci.media[i].stream.vid.enc_slot != PJSUA_INVALID_ID)
-	    {
-		med_idx = i;
-		break;
-	    }
-	}
-	if (med_idx < 0) {
-    	    PJSUA2_RAISE_ERROR3(PJ_ENOTFOUND, "getEncodingVideoMedia()",
-				"no active encoding video media");
-	}
+        for (unsigned i = 0; i < pj_ci.media_cnt; ++i) {
+            if (pj_ci.media[i].type == PJMEDIA_TYPE_VIDEO &&
+                pj_ci.media[i].stream.vid.enc_slot != PJSUA_INVALID_ID)
+            {
+                med_idx = i;
+                break;
+            }
+        }
+        if (med_idx < 0) {
+            PJSUA2_RAISE_ERROR3(PJ_ENOTFOUND, "getEncodingVideoMedia()",
+                                "no active encoding video media");
+        }
     }
 
     if (med_idx >= (int)pj_ci.media_cnt) {
-    	PJSUA2_RAISE_ERROR3(PJ_EINVAL, "getEncodingVideoMedia()",
-			    "invalid media index");
+        PJSUA2_RAISE_ERROR3(PJ_EINVAL, "getEncodingVideoMedia()",
+                            "invalid media index");
     }
     if (pj_ci.media[med_idx].type != PJMEDIA_TYPE_VIDEO) {
-    	PJSUA2_RAISE_ERROR3(PJ_EINVAL, "getEncodingVideoMedia()",
-			    "media is not video");
+        PJSUA2_RAISE_ERROR3(PJ_EINVAL, "getEncodingVideoMedia()",
+                            "media is not video");
     }
     if (pj_ci.media[med_idx].stream.vid.enc_slot == PJSUA_INVALID_ID) {
-    	PJSUA2_RAISE_ERROR3(PJ_EINVAL, "getEncodingVideoMedia()",
-			    "no encoding slot (recvonly?)");
+        PJSUA2_RAISE_ERROR3(PJ_EINVAL, "getEncodingVideoMedia()",
+                            "no encoding slot (recvonly?)");
     }
 
     VideoMediaHelper vm;
@@ -581,31 +684,31 @@ VideoMedia Call::getDecodingVideoMedia(int med_idx) const PJSUA2_THROW(Error)
     pjsua_call_get_info(id, &pj_ci);
 
     if (med_idx < 0) {
-	for (unsigned i = 0; i < pj_ci.media_cnt; ++i) {
-	    if (pj_ci.media[i].type == PJMEDIA_TYPE_VIDEO &&
-		pj_ci.media[i].stream.vid.dec_slot != PJSUA_INVALID_ID)
-	    {
-		med_idx = i;
-		break;
-	    }
-	}
-	if (med_idx < 0) {
-    	    PJSUA2_RAISE_ERROR3(PJ_ENOTFOUND, "getDecodingVideoMedia()",
-				"no active decoding video media");
-	}
+        for (unsigned i = 0; i < pj_ci.media_cnt; ++i) {
+            if (pj_ci.media[i].type == PJMEDIA_TYPE_VIDEO &&
+                pj_ci.media[i].stream.vid.dec_slot != PJSUA_INVALID_ID)
+            {
+                med_idx = i;
+                break;
+            }
+        }
+        if (med_idx < 0) {
+            PJSUA2_RAISE_ERROR3(PJ_ENOTFOUND, "getDecodingVideoMedia()",
+                                "no active decoding video media");
+        }
     }
 
     if (med_idx >= (int)pj_ci.media_cnt) {
-    	PJSUA2_RAISE_ERROR3(PJ_EINVAL, "getDecodingVideoMedia()",
-			    "invalid media index");
+        PJSUA2_RAISE_ERROR3(PJ_EINVAL, "getDecodingVideoMedia()",
+                            "invalid media index");
     }
     if (pj_ci.media[med_idx].type != PJMEDIA_TYPE_VIDEO) {
-    	PJSUA2_RAISE_ERROR3(PJ_EINVAL, "getDecodingVideoMedia()",
-			    "media is not video");
+        PJSUA2_RAISE_ERROR3(PJ_EINVAL, "getDecodingVideoMedia()",
+                            "media is not video");
     }
     if (pj_ci.media[med_idx].stream.vid.dec_slot == PJSUA_INVALID_ID) {
-    	PJSUA2_RAISE_ERROR3(PJ_EINVAL, "getDecodingVideoMedia()",
-			    "no decoding slot (sendonly?)");
+        PJSUA2_RAISE_ERROR3(PJ_EINVAL, "getDecodingVideoMedia()",
+                            "no decoding slot (sendonly?)");
     }
 
     VideoMediaHelper vm;
@@ -645,7 +748,7 @@ pj_stun_nat_type Call::getRemNatType() PJSUA2_THROW(Error)
 }
 
 void Call::makeCall(const string &dst_uri, const CallOpParam &prm)
-		    PJSUA2_THROW(Error)
+                    PJSUA2_THROW(Error)
 {
     pj_str_t pj_dst_uri = str2Pj(dst_uri);
     call_param param(prm.txOption, prm.opt, prm.reason);
@@ -658,16 +761,16 @@ void Call::makeCall(const string &dst_uri, const CallOpParam &prm)
 void Call::answer(const CallOpParam &prm) PJSUA2_THROW(Error)
 {
     call_param param(prm.txOption, prm.opt, prm.reason,
-    		     sdp_pool, prm.sdp.wholeSdp);
+                     sdp_pool, prm.sdp.wholeSdp);
     
     if (param.sdp) {
-    	PJSUA2_CHECK_EXPR( pjsua_call_answer_with_sdp(id, param.sdp,
-						      param.p_opt,
-    						      prm.statusCode,
-                                              	      param.p_reason,
-                                              	      param.p_msg_data) );
+        PJSUA2_CHECK_EXPR( pjsua_call_answer_with_sdp(id, param.sdp,
+                                                      param.p_opt,
+                                                      prm.statusCode,
+                                                      param.p_reason,
+                                                      param.p_msg_data) );
     } else {
-    	PJSUA2_CHECK_EXPR( pjsua_call_answer2(id, param.p_opt, prm.statusCode,
+        PJSUA2_CHECK_EXPR( pjsua_call_answer2(id, param.p_opt, prm.statusCode,
                                               param.p_reason,
                                               param.p_msg_data) );
     }
@@ -686,7 +789,7 @@ void Call::setHold(const CallOpParam &prm) PJSUA2_THROW(Error)
     call_param param(prm.txOption, prm.opt, prm.reason);
     
     PJSUA2_CHECK_EXPR( pjsua_call_set_hold2(id, prm.options,
-					    param.p_msg_data) );
+                                            param.p_msg_data) );
 }
 
 void Call::reinvite(const CallOpParam &prm) PJSUA2_THROW(Error)
@@ -694,7 +797,7 @@ void Call::reinvite(const CallOpParam &prm) PJSUA2_THROW(Error)
     call_param param(prm.txOption, prm.opt, prm.reason);
 
     PJSUA2_CHECK_EXPR( pjsua_call_reinvite2(id, param.p_opt,
-					    param.p_msg_data) );
+                                            param.p_msg_data) );
 }
 
 void Call::update(const CallOpParam &prm) PJSUA2_THROW(Error)
@@ -702,11 +805,11 @@ void Call::update(const CallOpParam &prm) PJSUA2_THROW(Error)
     call_param param(prm.txOption, prm.opt, prm.reason);
     
     PJSUA2_CHECK_EXPR( pjsua_call_update2(id, param.p_opt,
-					  param.p_msg_data) );
+                                          param.p_msg_data) );
 }
 
 void Call::xfer(const string &dest, const CallOpParam &prm)
-		PJSUA2_THROW(Error)
+                PJSUA2_THROW(Error)
 {
     call_param param(prm.txOption);
     pj_str_t pj_dest = str2Pj(dest);
@@ -721,7 +824,7 @@ void Call::xferReplaces(const Call& dest_call,
     
     PJSUA2_CHECK_EXPR(pjsua_call_xfer_replaces(id, dest_call.getId(),
                                                prm.options,
-					       param.p_msg_data) );
+                                               param.p_msg_data) );
 }
 
 void Call::processRedirect(pjsip_redirect_op cmd) PJSUA2_THROW(Error)
@@ -743,6 +846,15 @@ void Call::sendDtmf(const CallSendDtmfParam &param) PJSUA2_THROW(Error)
     PJSUA2_CHECK_EXPR(pjsua_call_send_dtmf(id, &pj_param));
 }
 
+void Call::sendText(const CallSendTextParam &param) PJSUA2_THROW(Error)
+{
+    pjsua_call_send_text_param pj_param;
+
+    pjsua_call_send_text_param_default(&pj_param);
+    pj_param.text = str2Pj(param.text);
+
+    PJSUA2_CHECK_EXPR(pjsua_call_send_text(id, &pj_param));
+}
 
 void Call::sendInstantMessage(const SendInstantMessageParam& prm)
     PJSUA2_THROW(Error)
@@ -772,7 +884,7 @@ void Call::sendRequest(const CallSendRequestParam &prm) PJSUA2_THROW(Error)
     call_param param(prm.txOption);
     
     PJSUA2_CHECK_EXPR(pjsua_call_send_request(id, &method,
-					      param.p_msg_data) );
+                                              param.p_msg_data) );
 }
 
 string Call::dump(bool with_media, const string indent) PJSUA2_THROW(Error)
@@ -812,7 +924,7 @@ bool Call::vidStreamIsRunning(int med_idx, pjmedia_dir dir) const
 
 void Call::vidSetStream(pjsua_call_vid_strm_op op,
                         const CallVidSetStreamParam &param)
-			PJSUA2_THROW(Error)
+                        PJSUA2_THROW(Error)
 {
 #if PJSUA_HAS_VIDEO
     pjsua_call_vid_strm_op_param prm;
@@ -828,6 +940,28 @@ void Call::vidSetStream(pjsua_call_vid_strm_op op,
 #endif
 }
 
+void Call::vidStreamModifyCodecParam(int med_idx, const VidCodecParam &param)
+                                     PJSUA2_THROW(Error)
+{
+#if PJSUA_HAS_VIDEO
+    pjmedia_vid_codec_param prm = param.toPj();
+    PJSUA2_CHECK_EXPR( pjsua_call_vid_stream_modify_codec_param(id, med_idx,
+                                                                &prm) );
+#else
+    PJ_UNUSED_ARG(med_idx);
+    PJ_UNUSED_ARG(param);
+    PJSUA2_RAISE_ERROR(PJ_EINVALIDOP);
+#endif
+}
+
+void Call::audStreamModifyCodecParam(int med_idx, const CodecParam &param)
+                                     PJSUA2_THROW(Error)
+{
+    pjmedia_codec_param prm = param.toPj();
+    PJSUA2_CHECK_EXPR( pjsua_call_aud_stream_modify_codec_param(id, med_idx,
+                                                                &prm) );
+}
+
 StreamInfo Call::getStreamInfo(unsigned med_idx) const PJSUA2_THROW(Error)
 {
     pjsua_stream_info pj_si;
@@ -866,16 +1000,16 @@ void Call::processMediaUpdate(OnCallMediaStateParam &prm)
     unsigned mi;
     
     if (pjsua_call_get_info(id, &pj_ci) == PJ_SUCCESS) {
-	if (medias.size()) {
-	    /* Clear medias. */
-	    for (mi = 0; mi < medias.size(); mi++) {
-		if (medias[mi]) {
-		    Endpoint::instance().mediaRemove((AudioMedia&)*medias[mi]);
-		    delete medias[mi];
-		}
-	    }
-	    medias.clear();	
-	}
+        if (medias.size()) {
+            /* Clear medias. */
+            for (mi = 0; mi < medias.size(); mi++) {
+                if (medias[mi]) {
+                    Endpoint::instance().mediaRemove((AudioMedia&)*medias[mi]);
+                    delete medias[mi];
+                }
+            }
+            medias.clear();     
+        }
 
         for (mi = 0; mi < pj_ci.media_cnt; mi++) {
             if (mi >= medias.size()) {
@@ -890,14 +1024,14 @@ void Call::processMediaUpdate(OnCallMediaStateParam &prm)
                 AudioMediaHelper *aud_med = (AudioMediaHelper*)medias[mi];
                 aud_med->setPortId(pj_ci.media[mi].stream.aud.conf_slot);
 
-		/* Add media if the conference slot ID is valid. */
+                /* Add media if the conference slot ID is valid. */
                 if (pj_ci.media[mi].stream.aud.conf_slot != PJSUA_INVALID_ID)
                 {
                     Endpoint::instance().mediaAdd((AudioMedia &)*aud_med);
                 } else {
                     Endpoint::instance().mediaRemove((AudioMedia &)*aud_med);
                 }
-	    }
+            }
         }
     }
     
@@ -913,35 +1047,35 @@ void Call::processStateChange(OnCallStateParam &prm)
     if (pjsua_call_get_info(id, &pj_ci) == PJ_SUCCESS &&
         pj_ci.state == PJSIP_INV_STATE_DISCONNECTED)
     {
-    	pjsua_call *call = &pjsua_var.calls[id];
+        pjsua_call *call = &pjsua_var.calls[id];
 
-	/* We are going to remove the Call object association below,
-	 * so we need to call onStreamDestroyed() callback here.
-	 */
-    	for (mi = 0; mi < call->med_cnt; ++mi) {
-    	    pjsua_call_media *call_med = &call->media[mi];
-	    if (call_med->type == PJMEDIA_TYPE_AUDIO &&
-	    	call_med->strm.a.stream)
-	    {
-    		OnStreamDestroyedParam strm_prm;
-    		strm_prm.stream = call_med->strm.a.stream;
-    		strm_prm.streamIdx = mi;
+        /* We are going to remove the Call object association below,
+         * so we need to call onStreamDestroyed() callback here.
+         */
+        for (mi = 0; mi < call->med_cnt; ++mi) {
+            pjsua_call_media *call_med = &call->media[mi];
+            if (call_med->type == PJMEDIA_TYPE_AUDIO &&
+                call_med->strm.a.stream)
+            {
+                OnStreamDestroyedParam strm_prm;
+                strm_prm.stream = call_med->strm.a.stream;
+                strm_prm.streamIdx = mi;
     
-    		onStreamDestroyed(strm_prm);	    	
-	    }
-    	}
+                onStreamDestroyed(strm_prm);            
+            }
+        }
 
         /* Clear medias. */
         for (mi = 0; mi < medias.size(); mi++) {
             if (medias[mi]) {
-		Endpoint::instance().mediaRemove((AudioMedia &)*medias[mi]);
+                Endpoint::instance().mediaRemove((AudioMedia &)*medias[mi]);
                 delete medias[mi];
             }
         }
         medias.clear();
 
-	/* Remove this Call object association */
-	pjsua_call_set_user_data(id, NULL);
+        /* Remove this Call object association */
+        pjsua_call_set_user_data(id, NULL);
     }
     
     onCallState(prm);
